import theano
import theano.tensor as T
import numpy as np

from neupy.utils import asfloat
from neupy.core.properties import ProperFractionProperty, NumberProperty
from .base import MinibatchGradientDescent


__all__ = ('Adam',)


class Adam(MinibatchGradientDescent):
    """ Adam algorithm.

    Parameters
    ----------
    beta1 : float
        Decay rate. Value need to be between ``0`` and ``1``.
        Defaults to ``0.95``.
    beta2 : float
        Decay rate. Value need to be between ``0`` and ``1``.
        Defaults to ``0.95``.
    epsilon : float
        Value need to be greater than ``0``. Defaults to ``1e-5``.
    step : float
        Learning rate, defaults to ``0.001``.
    {MinibatchGradientDescent.batch_size}
    {GradientDescent.addons}
    {ConstructableNetwork.connection}
    {ConstructableNetwork.error}
    {BaseNetwork.show_epoch}
    {BaseNetwork.shuffle_data}
    {BaseNetwork.epoch_end_signal}
    {BaseNetwork.train_end_signal}
    {Verbose.verbose}

    Methods
    -------
    {BaseSkeleton.predict}
    {SupervisedLearning.train}
    {BaseSkeleton.fit}
    """
    step = NumberProperty(default=0.001, minval=0)
    beta1 = ProperFractionProperty(default=0.9)
    beta2 = ProperFractionProperty(default=0.999)
    epsilon = NumberProperty(default=1e-7, minval=0)

    def init_layers(self):
        super(Adam, self).init_layers()
        for layer in self.layers:
            for parameter in layer.parameters:
                parameter_shape = T.shape(parameter).eval()
                parameter.prev_first_moment = theano.shared(
                    name="prev_first_moment_" + parameter.name,
                    value=asfloat(np.zeros(parameter_shape)),
                )
                parameter.prev_second_moment = theano.shared(
                    name="prev_second_moment_" + parameter.name,
                    value=asfloat(np.zeros(parameter_shape)),
                )

    def init_param_updates(self, layer, parameter):
        epoch = self.variables.epoch
        prev_first_moment = parameter.prev_first_moment
        prev_second_moment = parameter.prev_second_moment

        step = asfloat(self.variables.step)
        beta1 = asfloat(self.beta1)
        beta2 = asfloat(self.beta2)
        epsilon = asfloat(self.epsilon)

        gradient = T.grad(self.variables.error_func, wrt=parameter)

        first_moment = (
            beta1 * prev_first_moment +
            asfloat(1. - beta1) * gradient)
        second_moment = (
            beta2 * prev_second_moment +
            asfloat(1. - beta2) * gradient ** 2
        )

        first_moment_bias_corrected = first_moment / (1. - beta1 ** epoch)
        second_moment_bias_corrected = second_moment / (1. - beta2 ** epoch)

        parameter_delta = first_moment_bias_corrected * (
            T.sqrt(second_moment_bias_corrected) + epsilon
        )

        return [
            (prev_first_moment, first_moment),
            (prev_second_moment, second_moment),
            (parameter, parameter - step * parameter_delta),
        ]



import numpy as np

from neupy.layers.connections import NetworkConnectionError
from neupy.layers.base import BaseLayer
from neupy.core.properties import (TypedListProperty, IntProperty,
                                   ProperFractionProperty)


__all__ = ('Output', 'CompetitiveOutput', 'StepOutput', 'RoundedOutput',
           'ArgmaxOutput')


class Output(BaseLayer):
    """ Simples output layer class which does not make any transformations.
    Output from this layer is the same as input.

    Parameters
    ----------
    size : int
        Network output size.
    """
    size = IntProperty(required=True, minval=1)

    def __init__(self, size, **options):
        options['size'] = size
        super(Output, self).__init__(**options)

    def initialize(self):
        pass

    def relate_to(self, right_layer):
        raise NetworkConnectionError("Can't create connection "
                                     "from output layer")

    def output(self, value):
        return value

    def __repr__(self):
        return '{name}({size})'.format(name=self.__class__.__name__,
                                       size=self.size)


class CompetitiveOutput(Output):
    """ Competitive layer output. Layer output will return the result where
    all zero values and one value which has greatest value will be one.

    Parameters
    ----------
    {Output.size}
    """
    def output(self, value):
        output = np.zeros(value.shape, dtype=np.int0)
        max_args = value.argmax(axis=1)
        output[range(value.shape[0]), max_args] = 1
        return output


class StepOutput(Output):
    """ The behaviour for this layer is the same as for step function.

    Parameters
    ----------
    output_bounds : tuple
        Value is must be a tuple which contains two elements where first one
        identify lower output value and the second one - bigger. Defaults
        to ``(0, 1)``.
    critical_point : float
        Critical point is set up step function bias. Value equal to this
        point should be equal to the lower bound. Defaults to ``0``.
    {Output.size}
    """
    output_bounds = TypedListProperty(default=(0, 1))
    critical_point = ProperFractionProperty(default=0)

    def output(self, value):
        lower_bound, upper_bound = self.output_bounds
        return np.where(value <= self.critical_point,
                        lower_bound, upper_bound)


class RoundedOutput(Output):
    """ Round output layer value.

    Parameters
    ----------
    decimals : int
        The precision in decimal digits for output value.
    {Output.size}
    """
    decimals = IntProperty(default=0, minval=0)

    def output(self, value):
        return np.round(value, self.decimals)


class ArgmaxOutput(Output):
    """ Return number of feature that have maximum value for each sample.

    Parameters
    ----------
    decimals : int
    The precision in decimal digits for output value.
    {Output.size}
    """
    def output(self, value):
        return value.argmax(axis=1)



__all__ = ('SINGLE_STEP_UPDATE', 'MULTIPLE_STEP_UPDATE', 'WEIGHT_PENALTY',
           'addon_types', 'StepSelectionBuiltIn', 'NoMultipleStepSelection')


# Available add-on types
SINGLE_STEP_UPDATE = 1
MULTIPLE_STEP_UPDATE = 2
WEIGHT_PENALTY = 3


addon_types = {
    SINGLE_STEP_UPDATE: "Single-step update",
    MULTIPLE_STEP_UPDATE: "Multi-step update",
    WEIGHT_PENALTY: "Weight penalty",
}


class StepSelectionBuiltIn(object):
    """ Mixin excludes add-ons that modify learning rate.
    """
    supported_addon_types = [WEIGHT_PENALTY]


class NoMultipleStepSelection(object):
    """ Mixin excludes add-ons that use multiple learning rates for
    one neural network.
    """
    supported_addon_types = [SINGLE_STEP_UPDATE, WEIGHT_PENALTY]


class NoStepSelection(StepSelectionBuiltIn):
    """ Mixin that excludes step property from neural network
    class.
    """
    def init_properties(self):
        del self.step
        super(NoStepSelection, self).init_properties()



from neupy.core.properties import IntProperty
from .base import SingleStepConfigurable


__all__ = ('SimpleStepMinimization',)


class SimpleStepMinimization(SingleStepConfigurable):
    """ Algorithm minimizes learning step monotonically after
    each iteration.

    Parameters
    ----------
    epochs_step_minimizator : int
        The parameter controls the frequency reduction step with respect
        to epochs. Defaults to ``100`` epochs. Can't be less than ``1``.
        Less value mean that step decrease faster.

    Warns
    -----
    {SingleStepConfigurable.Warns}

    Examples
    --------
    >>> from neupy import algorithms
    >>>
    >>> bpnet = algorithms.GradientDescent(
    ...     (2, 4, 1),
    ...     step=0.1,
    ...     verbose=False,
    ...     addons=[algorithms.SimpleStepMinimization]
    ... )
    >>>

    See Also
    --------
    :network:`SearchThenConverge`
    """
    epochs_step_minimizator = IntProperty(minval=1, default=100)

    def init_train_updates(self):
        updates = super(SimpleStepMinimization, self).init_train_updates()
        epoch = self.variables.epoch
        step = self.variables.step

        step_update_condition = self.step / (
            1 + epoch / self.epochs_step_minimizator
        )
        updates.extend([
            (step, step_update_condition),
        ])
        return updates



import numpy as np
from numpy.core.umath_tests import inner1d


__all__ = ('sign2bin', 'bin2sign', 'hopfield_energy')


def sign2bin(matrix):
    """ Convert -1 values in sign binary matrix to binary values.

    Parameters
    ----------
    matrix : array-like

    Returns
    -------
    array-like
        Return they same matrix with modified values.
    """
    return np.where(matrix == 1, 1, 0)


def bin2sign(matrix):
    """ Convert zeros in banary matrix to -1 values.

    Parameters
    ----------
    matrix : array-like

    Returns
    -------
    array-like
        Return they same matrix with modified values.
    """
    return np.where(matrix == 0, -1, 1)


def hopfield_energy(weight, input_data, output_data):
    """ Compute Hopfield energy between input data, output data and
    neural network weights.

    Parameters
    ----------
    input_data : vector
    output_data : vector
    weight : 2d array-like

    Returns
    -------
    float
        Hopfield energy for specific data and weights.
    """
    return -0.5 * inner1d(input_data.dot(weight), output_data)



from __future__ import print_function

import time
import textwrap
from operator import attrgetter
from abc import abstractmethod

import numpy as np
from six import with_metaclass

from neupy.core.docs import SharedDocs, SharedDocsABCMeta


__all__ = ("TableBuilder", "Column", "TimeColumn", "NumberColumn",
           "TableDrawingError")


class TableDrawingError(AttributeError):
    """ Exception specific for ``TableBuilder`` class functionality.
    """


class Column(SharedDocs):
    """ Simple column class that helps discribe structure for
    ``TableBuilder`` class instance.

    Parameters
    ----------
    name : str
        Column name. Value would be displayed in header. In case when
        ``width`` parameter equal to ``None``, string width will identify
        column width.
    dtype : object
        Column data format. Defaults to ``str``.
    width : int or None
        Column width. Defaults to ``None``.
    """

    def __init__(self, name, dtype=str, width=None):
        if width is None:
            width = len(name)

        self.name = name
        self.dtype = dtype
        self.width = width

    def format_value(self, value):
        """ Convert input value to specified type

        Parameters
        ----------
        value : object

        Returns
        -------
        object
            Function return converted input value to specified
            data type.
        """
        return self.dtype(value)


def format_time(value):
    """ Convert seconds to the value format that easy
    to understand.

    Parameters
    ----------
    value : float
        Time interval in seconds.

    Returns
    -------
    str

    Examples
    --------
    >>> col = TimeColumn("Time")
    >>> col.format_value(0.001)
    '1 ms'
    >>> col.format_value(0.5)
    '0.5 sec'
    >>> col.format_value(1.5)
    '1.5 sec'
    >>> col.format_value(15)
    '00:00:15'
    >>> col.format_value(15045)
    '04:10:45'
    """
    if value < 0.05:
        return "{} ms".format(round(value * 10 ** 3))

    elif value < 10:
        return "{} sec".format(round(value, 1))

    return time.strftime("%H:%M:%S", time.gmtime(value))


class TimeColumn(Column):
    """ Columns useful for time formating from seconds to more
    informative and readable format.

    Parameters
    ----------
    {Column.name}
    {Column.dtype}
    {Column.width}
    """

    def format_value(self, value):
        return format_time(value)


class NumberColumn(Column):
    """ Class describe float column type.

    Parameters
    ----------
    places : int
        Float number rounding precision. Defaults to ``6``.
    {Column.name}
    {Column.dtype}
    {Column.width}
    """

    def __init__(self, places=6, *args, **kwargs):
        super(NumberColumn, self).__init__(*args, **kwargs)
        self.places = places

    def format_value(self, value):
        """ Round a number to a given precision in decimal digits

        Parameters
        ----------
        value : float

        Returns
        -------
        float
            Rounded input value.
        """
        if not isinstance(value, (int, float, np.floating, np.integer)):
            return value

        if value > 100:
            return "~{:.0f}".format(value)

        return "{value:.{places}f}".format(value=value,
                                           places=self.places)


class BaseState(with_metaclass(SharedDocsABCMeta)):
    """ Base abstract class that identify all important methods for
    ``TableBuilder`` class states.

    Parameters
    ----------
    table : TableBuilder instance
        Accept summary table instance. State is able to control
        properties from main ``TableBuilder`` class instantance
    """

    def __init__(self, table):
        self.table = table

    def line(self):
        """ Draw ASCII line. Line width depence on the table
        column sizes.
        """
        self.table.stdout('\r' + '-' * self.table.total_width)

    def message(self, text):
        """ Write additional message in table. All seperators
        between columns will be ignored.
        """
        self.line()
        # Excluding from the total width 2 symbols related to
        # the separators near the table edges and 2 symbols
        # related to the spaces near these edges
        max_line_width = self.table.total_width - 4

        for text_row in textwrap.wrap(text, max_line_width):
            formated_text = text_row.ljust(max_line_width)
            self.table.stdout("\r| " + formated_text + " |")

        self.line()

    @abstractmethod
    def start(self):
        pass

    @abstractmethod
    def finish(self):
        pass

    @abstractmethod
    def header(self):
        pass

    @abstractmethod
    def row(self, data):
        pass


class DrawingState(BaseState):
    """ Identify active state for ``TableBuilder`` class instance.
    In this state summary table instance is able to show information
    in terminal.

    Parameters
    ----------
    {BaseState.table}
    """

    def start(self):
        raise TableDrawingError("Table drawing already started")

    def finish(self):
        self.line()
        self.table.state = IdleState(self.table)

    def header(self):
        raise TableDrawingError("Header already drawn")

    def row(self, data):
        formated_data = []
        for val, column in zip(data, self.table.columns):
            val = column.format_value(val)
            cell_value = str(val).ljust(column.width)
            formated_data.append(cell_value)
        self.table.stdout("\r| " + " | ".join(formated_data) + " |")


class IdleState(BaseState):
    """ Identify idle state for ``TableBuilder`` class instance.
    In this state summary table instance isn't able to show information
    in terminal.

    Parameters
    ----------
    {BaseState.table}
    """

    def start(self):
        self.header()
        self.table.state = DrawingState(self.table)

    def finish(self):
        raise TableDrawingError("Table drawing already finished or "
                                "didn't started")

    def header(self):
        self.line()

        headers = []
        for column in self.table.columns:
            header_name = str(column.name).ljust(column.width)
            headers.append(header_name)
        self.table.stdout("\r| " + " | ".join(headers) + " |")

        self.line()

    def row(self, data):
        raise TableDrawingError("Table drawing already finished or "
                                "didn't started")


class TableBuilder(SharedDocs):
    """ Build ASCII tables using simple structure.

    Parameters
    ----------
    *columns
        Table structure. Accept ``Column`` instance classes.
    stdout : func
        Function through which the message will be transmitted.
    """
    def __init__(self, *columns, **kwargs):
        valid_kwargs = ['stdout']
        # In Python 2 doesn't work syntax like
        # def __init__(self, *columns, stdout=print):
        # Code below implements the same.
        stdout = kwargs.get('stdout', print)

        if any(kwarg not in valid_kwargs for kwarg in kwargs):
            raise ValueError("Invalid keyword arguments. Available "
                             "only: {}".format(valid_kwargs))

        for column in columns:
            if not isinstance(column, Column):
                raise TypeError("Column should be ``Column`` class "
                                "instance.")

        self.columns = columns
        self.stdout = stdout
        self.state = IdleState(self)

        text_width = sum(map(attrgetter('width'), columns))
        n_columns = len(columns)
        n_separators = n_columns + 1
        n_margins = 2 * n_columns

        self.total_width = text_width + n_separators + n_margins

    def __getattr__(self, attr):
        if attr not in self.__dict__:
            return getattr(self.state, attr)
        return super(TableBuilder, self).__getattr__(attr)



import theano
import theano.tensor as T
from theano.ifelse import ifelse
import numpy as np

from neupy.utils import asfloat
from neupy.network import errors
from neupy.core.properties import BoundedProperty, ChoiceProperty
from neupy.algorithms import GradientDescent
from neupy.algorithms.gd import NoStepSelection
from neupy.algorithms.utils import (parameters2vector, iter_parameters,
                                    setup_parameter_updates)


__all__ = ('LevenbergMarquardt',)


def compute_jaccobian(errors, parameters):
    """ Compute Jacobbian.

    Parameters
    ----------
    errors : Theano variable
        Computed MSE for each sample separetly.
    parameters : list of Theano variable
        Neural network parameters (e.g. weights, biases).

    Returns
    -------
    Theano variable
    """
    n_samples = errors.shape[0]

    def find_jacobbian(i, errors, *params):
        return T.grad(T.sum(errors[i]), wrt=params)

    J, _ = theano.scan(
        find_jacobbian,
        sequences=T.arange(n_samples),
        non_sequences=[errors] + parameters
    )

    jaccobians = []
    for jaccobian, parameter in zip(J, parameters):
        jaccobian = jaccobian.reshape((n_samples, parameter.size))
        jaccobians.append(jaccobian)

    return T.concatenate(jaccobians, axis=1)


class LevenbergMarquardt(NoStepSelection, GradientDescent):
    """ Levenberg-Marquardt algorithm.

    Notes
    -----
    * Network minimizes only Mean Squared Error function.

    Parameters
    ----------
    mu : float
        Control invertion for J.T * J matrix, defaults to `0.1`.
    mu_update_factor : float
        Factor to decrease the mu if update decrese the error, otherwise
        increse mu by the same factor.
    error: {{'mse'}}
        Levenberg-Marquardt works only for quadratic functions.
        Defaults to ``mse``.
    {GradientDescent.addons}
    {ConstructableNetwork.connection}
    {BaseNetwork.step}
    {BaseNetwork.show_epoch}
    {BaseNetwork.shuffle_data}
    {BaseNetwork.epoch_end_signal}
    {BaseNetwork.train_end_signal}
    {Verbose.verbose}

    Methods
    -------
    {BaseSkeleton.predict}
    {SupervisedLearning.train}
    {BaseSkeleton.fit}

    Examples
    --------
    Simple example

    >>> import numpy as np
    >>> from neupy import algorithms
    >>>
    >>> x_train = np.array([[1, 2], [3, 4]])
    >>> y_train = np.array([[1], [0]])
    >>>
    >>> lmnet = algorithms.LevenbergMarquardt(
    ...     (2, 3, 1),
    ...     verbose=False
    ... )
    >>> lmnet.train(x_train, y_train)

    Diabets dataset example

    >>> import numpy as np
    >>> from sklearn import datasets, preprocessing
    >>> from sklearn.cross_validation import train_test_split
    >>> from neupy import algorithms, layers
    >>> from neupy.estimators import rmsle
    >>>
    >>> dataset = datasets.load_diabetes()
    >>> data, target = dataset.data, dataset.target
    >>>
    >>> data_scaler = preprocessing.MinMaxScaler()
    >>> target_scaler = preprocessing.MinMaxScaler()
    >>>
    >>> x_train, x_test, y_train, y_test = train_test_split(
    ...     data_scaler.fit_transform(data),
    ...     target_scaler.fit_transform(target),
    ...     train_size=0.85
    ... )
    >>>
    >>> # Network
    ... lmnet = algorithms.LevenbergMarquardt(
    ...     connection=[
    ...         layers.Sigmoid(10),
    ...         layers.Sigmoid(40),
    ...         layers.Output(1),
    ...     ],
    ...     mu_update_factor=2,
    ...     mu=0.1,
    ...     step=0.25,
    ...     show_epoch=10,
    ...     use_bias=False,
    ...     verbose=False
    ... )
    >>> lmnet.train(x_train, y_train, epochs=100)
    >>> y_predict = lmnet.predict(x_test)
    >>>
    >>> error = rmsle(target_scaler.inverse_transform(y_test),
    ...               target_scaler.inverse_transform(y_predict).round())
    >>> error
    0.47548200957888398

    See Also
    --------
    :network:`GradientDescent` : GradientDescent algorithm.
    """

    mu = BoundedProperty(default=0.01, minval=0)
    mu_update_factor = BoundedProperty(default=5, minval=1)
    error = ChoiceProperty(default='mse', choices={'mse': errors.mse})

    def init_variables(self):
        super(LevenbergMarquardt, self).init_variables()
        self.variables.update(
            mu=theano.shared(name='mu', value=asfloat(self.mu)),
            last_error=theano.shared(name='last_error', value=np.nan),
        )

    def init_train_updates(self):
        network_output = self.variables.network_output
        prediction_func = self.variables.train_prediction_func
        last_error = self.variables.last_error
        error_func = self.variables.error_func
        mu = self.variables.mu

        new_mu = ifelse(
            T.lt(last_error, error_func),
            mu * self.mu_update_factor,
            mu / self.mu_update_factor,
        )

        mse_for_each_sample = T.mean(
            (network_output - prediction_func) ** 2,
            axis=1
        )

        params = list(iter_parameters(self))
        param_vector = parameters2vector(self)

        J = compute_jaccobian(mse_for_each_sample, params)
        n_params = J.shape[1]

        updated_params = param_vector - T.nlinalg.matrix_inverse(
            J.T.dot(J) + new_mu * T.eye(n_params)
        ).dot(J.T).dot(mse_for_each_sample)

        updates = [(mu, new_mu)]
        parameter_updates = setup_parameter_updates(params, updated_params)
        updates.extend(parameter_updates)

        return updates

    def on_epoch_start_update(self, epoch):
        super(LevenbergMarquardt, self).on_epoch_start_update(epoch)

        last_error = self.errors.last()
        if last_error is not None:
            self.variables.last_error.set_value(last_error)



from numpy import dot

from neupy.utils import format_data
from neupy.core.properties import BoundedProperty
from neupy.network.base import BaseNetwork
from neupy.network.learning import LazyLearning
from .utils import pdf_between_data


__all__ = ('GRNN',)


class GRNN(LazyLearning, BaseNetwork):
    """ Generalized Regression Neural Network.

    Parameters
    ----------
    std : float
        standard deviation for PDF function, default to 0.1.
    {Verbose.verbose}

    Methods
    -------
    {LazyLearning.train}
    {BaseSkeleton.predict}
    {BaseSkeleton.fit}

    Examples
    --------
    >>> from sklearn import datasets
    >>> from sklearn.cross_validation import train_test_split
    >>> from neupy import algorithms, estimators, environment
    >>>
    >>> environment.reproducible()
    >>>
    >>> dataset = datasets.load_diabetes()
    >>> x_train, x_test, y_train, y_test = train_test_split(
    ...     dataset.data, dataset.target, train_size=0.7,
    ...     random_state=0
    ... )
    >>>
    >>> nw = algorithms.GRNN(std=0.1, verbose=False)
    >>> nw.train(x_train, y_train)
    >>> result = nw.predict(x_test)
    >>> estimators.rmsle(result, y_test)
    0.4245120142774001
    """
    std = BoundedProperty(default=0.1, minval=0)

    def train(self, input_train, target_train, copy=True):
        input_train = format_data(input_train, copy=copy)
        target_train = format_data(target_train, copy=copy)

        if target_train.shape[1] != 1:
            raise ValueError("Target value must be one dimentional array")

        LazyLearning.train(self, input_train, target_train)

    def predict(self, input_data):
        super(GRNN, self).predict(input_data)

        input_data = format_data(input_data)

        input_data_size = input_data.shape[1]
        train_data_size = self.input_train.shape[1]

        if input_data_size != train_data_size:
            raise ValueError("Input data must contains {0} features, got "
                             "{1}".format(train_data_size, input_data_size))

        ratios = pdf_between_data(self.input_train, input_data,
                                  self.std)
        return (dot(self.target_train.T, ratios) / ratios.sum(axis=0)).T



import six
import theano.tensor as T

from neupy.core.config import ConfigMeta
from .base import ParameterBasedLayer


__all__ = ('ActivationLayer', 'Linear', 'Sigmoid', 'HardSigmoid', 'Step',
           'Tanh', 'Relu', 'Softplus', 'Softmax')


class LayerMeta(ConfigMeta):
    """ Meta-class overrides activation functions an make them
    behave as usual functions.
    """
    def __new__(cls, clsname, bases, attrs):
        if 'activation_function' in attrs:
            # Override `activation_function` in `staticmethod` by default.
            # Python 2 translate any assigned function as class method
            # and try call it with with argument `self` which broke
            # logic. For this reason we try make it static.
            attrs['activation_function'] = staticmethod(
                attrs['activation_function']
            )
        return super(LayerMeta, cls).__new__(cls, clsname, bases, attrs)


class ActivationLayer(six.with_metaclass(LayerMeta, ParameterBasedLayer)):
    """ Base class for the layers based on the activation
    functions.

    Parameters
    ----------
    size : int or None
        Layer input size. ``None`` means that layer will not create
        parameters and will return only activation function
        output for the specified input value.
    {ParameterBasedLayer.weight}
    {ParameterBasedLayer.bias}
    {ParameterBasedLayer.init_method}
    {ParameterBasedLayer.bounds}
    """
    def __init__(self, size=None, **options):
        # If you set class method function variable, python will interpret
        # it as a new class method and will call it with a `self`
        # argument.
        if hasattr(self.__class__, 'activation_function'):
            self.activation_function = self.__class__.activation_function
        super(ActivationLayer, self).__init__(size, **options)

    def initialize(self):
        if self.size is not None:
            super(ActivationLayer, self).initialize()

    def output(self, input_value):
        if self.size is not None:
            input_value = T.dot(input_value, self.weight) + self.bias
        return self.activation_function(input_value)

    def __repr__(self):
        if self.size is None:
            return super(ParameterBasedLayer, self).__repr__()
        return super(ActivationLayer, self).__repr__()


class Linear(ActivationLayer):
    """ The layer with the linear activation function.

    Parameters
    ----------
    {ActivationLayer.size}
    {ParameterBasedLayer.weight}
    {ParameterBasedLayer.bias}
    {ParameterBasedLayer.init_method}
    {ParameterBasedLayer.bounds}
    """
    activation_function = (lambda x: x)


class Sigmoid(ActivationLayer):
    """ The layer with the sigmoid activation function.

    Parameters
    ----------
    {ActivationLayer.size}
    {ParameterBasedLayer.weight}
    {ParameterBasedLayer.bias}
    {ParameterBasedLayer.init_method}
    {ParameterBasedLayer.bounds}
    """
    activation_function = T.nnet.sigmoid


class HardSigmoid(ActivationLayer):
    """ The layer with the hard sigmoid activation function.

    Parameters
    ----------
    {ActivationLayer.size}
    {ParameterBasedLayer.weight}
    {ParameterBasedLayer.bias}
    {ParameterBasedLayer.init_method}
    {ParameterBasedLayer.bounds}
    """
    activation_function = T.nnet.hard_sigmoid


def step_function(value):
    """ Step activation function.

    Parameters
    ----------
    value : symbolic tensor
        Tensor to compute the activation function for.
    alpha : scalar or tensor, optional
        Slope for negative input, usually between 0 and 1. The
        default value of 0 will lead to the standard rectifier, 1 will lead to
        a linear activation function, and any value in between will give a
        leaky rectifier. A shared variable (broadcastable against `x`) will
        result in a parameterized rectifier with learnable slope(s).

    Returns
    -------
    symbolic tensor
        Element-wise rectifier applied to `x`.
    """
    return T.gt(value, 0)


class Step(ActivationLayer):
    """ The layer with the the step activation function.

    Parameters
    ----------
    {ActivationLayer.size}
    {ParameterBasedLayer.weight}
    {ParameterBasedLayer.bias}
    {ParameterBasedLayer.init_method}
    {ParameterBasedLayer.bounds}
    """
    activation_function = step_function


class Tanh(ActivationLayer):
    """ The layer with the `tanh` activation function.

    Parameters
    ----------
    {ActivationLayer.size}
    {ParameterBasedLayer.weight}
    {ParameterBasedLayer.bias}
    {ParameterBasedLayer.init_method}
    {ParameterBasedLayer.bounds}
    """
    activation_function = T.tanh


class Relu(ActivationLayer):
    """ The layer with the rectifier activation function.

    Parameters
    ----------
    {ActivationLayer.size}
    {ParameterBasedLayer.weight}
    {ParameterBasedLayer.bias}
    {ParameterBasedLayer.init_method}
    {ParameterBasedLayer.bounds}
    """
    activation_function = T.nnet.relu


class Softplus(ActivationLayer):
    """ The layer with the softplus activation function.

    Parameters
    ----------
    {ActivationLayer.size}
    {ParameterBasedLayer.weight}
    {ParameterBasedLayer.bias}
    {ParameterBasedLayer.init_method}
    {ParameterBasedLayer.bounds}
    """
    activation_function = T.nnet.softplus


class Softmax(ActivationLayer):
    """ The layer with the softmax activation function.

    Parameters
    ----------
    {ActivationLayer.size}
    {ParameterBasedLayer.weight}
    {ParameterBasedLayer.bias}
    {ParameterBasedLayer.init_method}
    {ParameterBasedLayer.bounds}
    """
    activation_function = T.nnet.softmax



""" Code from a `tqdm` library: https://github.com/noamraph/tqdm
"""
import sys
import time
import collections

import numpy as np


__all__ = ('progressbar',)


def format_interval(time_in_seconds):
    """ Format seconds into human readable format.

    Parameters
    ----------
    time_in_seconds : float

    Returns
    -------
    str
        Formated time.
    """
    mins, seconds = divmod(int(time_in_seconds), 60)
    hours, minutes = divmod(mins, 60)

    if hours > 0:
        return '{:0>2d}:{:0>2d}:{:0>2d}'.format(hours, minutes, seconds)

    return '{:0>2d}:{:0>2d}'.format(minutes, seconds)


def format_error(error):
    """ Format the error value.

    Parameters
    ----------
    error : float, list or None

    Returns
    -------
    str
        Formated error value.
    """
    if error is None:
        return '?'

    if isinstance(error, collections.Iterable):
        error = np.atleast_1d(error).item(0)

    return '{:.5f}'.format(error)


def format_meter(n_finished, n_total, elapsed, error=None):
    """ Format meter.
    Parameters
    ----------
    n_finished : int
        Number of finished iterations.
    n_total : int or None
        Total number of iterations.
    elapsed : int
        Number of seconds passed since start
    """
    if n_finished > n_total:
        n_total = None

    elapsed_str = format_interval(elapsed)

    if n_total is not None:
        frac = float(n_finished) / n_total

        n_bars = 10
        bar_length = int(frac * n_bars)
        bar = '#' * bar_length + '-' * (n_bars - bar_length)

        percentage = '%3d%%' % (frac * 100)

        if n_finished:
            left_str = format_interval(
                elapsed / n_finished * (n_total - n_finished)
            )
        else:
            left_str = '?'

        return '|{}| {}/{} {} [elapsed: {} left: {} error: {}]'.format(
            bar, n_finished, n_total, percentage,
            elapsed_str, left_str, format_error(error)
        )

    return '{} [elapsed: {}]'.format(n_finished, elapsed_str)


class StatusPrinter(object):
    """ Class is able to write and rewrite the last row in
    the terminal

    Parameters
    ----------
    file : file
        File object instance that was opened using
        write mode.
    """
    def __init__(self, file):
        self.file = file
        self.last_printed_len = 0

    def write(self, text):
        n_spaces = max(self.last_printed_len - len(text), 0)

        self.file.write('\r' + text + ' ' * n_spaces)
        self.file.flush()

        self.last_printed_len = len(text)

    def clean(self):
        self.write('')
        self.file.write('\r')


def progressbar(iterable, desc='', total=None, file=sys.stderr,
                mininterval=0.05, miniters=1):
    """ Get an iterable object, and return an iterator which acts
    exactly like the iterable, but prints a progress meter and updates
    it every time a value is requested.

    Parameters
    ----------
    desc : str
        Can contain a short string, describing the progress,
        that is added in the beginning of the line.
    total : int
        Can give the number of expected iterations. If not given,
        len(iterable) is used if it is defined.
    file : object
        Can be a file-like object to output the progress message to.
    mininterval : float
        Defaults to ``0.05``.
    miniters : int
        If less than mininterval seconds or miniters
        iterations have passed since the last progress meter
        update, it is not updated again. Defaults to ``1``.
    """
    if total is None:
        try:
            total = len(iterable)
        except TypeError:
            total = None

    prefix = desc + ': ' if desc else ''

    printer = StatusPrinter(file)
    status = prefix + format_meter(0, total, 0)
    printer.write(status)

    start_time = last_print_time = time.time()
    last_print_n = 0
    n = 0

    try:
        for obj in iterable:
            error = (yield)
            yield obj

            n += 1
            if n - last_print_n >= miniters:
                current_time = time.time()

                if (current_time - last_print_time) >= mininterval:
                    time_delta = current_time - start_time
                    formated_str = format_meter(n, total, time_delta, error)

                    printer.write(prefix + formated_str)

                    last_print_n = n
                    last_print_time = current_time
    finally:
        printer.clean()



import random

import theano
import numpy as np


__all__ = ('sandbox', 'reproducible')


def sandbox():
    theano.config.linker = 'py'
    theano.config.mode = 'FAST_COMPILE'
    theano.config.optimizer = 'fast_compile'
    theano.config.allow_gc = False


def reproducible(seed=0):
    np.random.seed(seed)
    random.seed(seed)



from neupy.core.properties import ProperFractionProperty, BoundedProperty
from .base import SingleStepConfigurable


__all__ = ('DeltaBarDelta',)


class DeltaBarDelta(SingleStepConfigurable):
    beta = ProperFractionProperty(default=0.5)
    increase_factor = BoundedProperty(default=0.1, minval=0)
    decrease_factor = ProperFractionProperty(default=0.9)



from itertools import chain

import numpy as np
import theano.tensor as T


__all__ = ('count_parameters', 'parameters2vector', 'iter_parameters',
           'setup_parameter_updates')


def iter_parameters(network):
    """ Iterate over all network parameters.

    Parameters
    ----------
    network : ConstructableNetwork instance

    Returns
    -------
    iterator
        Returns iterator that contains all weights and biases from the
        network. Parameters from the first layer will be at the beggining
        and the other will be in the same order as layers in the
        network.
    """
    parameters = [layer.parameters for layer in network.layers]
    return chain(*parameters)


def parameters2vector(network):
    """ Concatenate all network parameters in one big vector.

    Parameters
    ----------
    network : ConstructableNetwork instance

    Returns
    -------
    object
        Returns concatenated parameters in one big vector.
    """
    params = iter_parameters(network)
    return T.concatenate([param.flatten() for param in params])


def count_parameters(network):
    """ Count number of parameters in Neural Network.

    Parameters
    ----------
    network : ConstructableNetwork instance

    Returns
    -------
    int
        Number of parameters.
    """
    params = iter_parameters(network)
    return np.sum([param.get_value().size for param in params])


def setup_parameter_updates(parameters, parameter_update_vector):
    """ Creates update rules for list of parameters from one vector.
    Function is useful in Conjugate Gradient or
    Levenberg-Marquardt optimization algorithms

    Parameters
    ----------
    parameters : list
    parameter_update_vector : Theano varible

    Returns
    -------
    list
        List of updates separeted for each parameter.
    """
    updates = []
    start_position = 0

    for parameter in parameters:
        end_position = start_position + parameter.size

        new_parameter = T.reshape(
            parameter_update_vector[start_position:end_position],
            parameter.shape
        )
        updates.append((parameter, new_parameter))

        start_position = end_position

    return updates



from numpy import unique, zeros, dot, sum as np_sum

from neupy.utils import format_data
from neupy.core.properties import BoundedProperty
from neupy.network.base import BaseNetwork
from neupy.network.learning import LazyLearning

from .utils import pdf_between_data


__all__ = ('PNN',)


class PNN(LazyLearning, BaseNetwork):
    """ Probabilistic Neural Network for classification.

    Parameters
    ----------
    std : float
        standard deviation for PDF function, default to 0.1.
    {Verbose.verbose}

    Methods
    -------
    {LazyLearning.train}
    {BaseSkeleton.predict}
    {BaseSkeleton.fit}

    Examples
    --------
    >>> import numpy as np
    >>>
    >>> from sklearn import datasets
    >>> from sklearn import metrics
    >>> from sklearn.cross_validation import train_test_split
    >>> from neupy import algorithms, environment
    >>>
    >>> environment.reproducible()
    >>>
    >>> dataset = datasets.load_digits()
    >>> x_train, x_test, y_train, y_test = train_test_split(
    ...     dataset.data, dataset.target, train_size=0.7
    ... )
    >>>
    >>> nw = algorithms.PNN(std=10, verbose=False)
    >>> nw.train(x_train, y_train)
    >>> result = nw.predict(x_test)
    >>> metrics.accuracy_score(y_test, result)
    0.98888888888888893
    """
    std = BoundedProperty(default=0.1, minval=0)

    def __init__(self, **options):
        super(PNN, self).__init__(**options)
        self.classes = None

    def train(self, input_train, target_train, copy=True):
        input_train = format_data(input_train, copy=copy)
        target_train = format_data(target_train, copy=copy)

        LazyLearning.train(self, input_train, target_train)

        if target_train.shape[1] != 1:
            raise ValueError("Target value must be in 1 dimention")

        classes = self.classes = unique(target_train)
        number_of_classes = classes.size
        row_comb_matrix = self.row_comb_matrix = zeros(
            (number_of_classes, input_train.shape[0])
        )
        class_ratios = self.class_ratios = zeros(number_of_classes)

        for i, class_name in enumerate(classes):
            class_val_positions = (target_train == i)
            row_comb_matrix[i, class_val_positions.ravel()] = 1
            class_ratios[i] = np_sum(class_val_positions)

    def predict_proba(self, input_data):
        raw_output = self.predict_raw(input_data)

        total_output_sum = raw_output.sum(axis=0).reshape(
            (raw_output.shape[1], 1)
        )
        return raw_output.T / total_output_sum

    def predict_raw(self, input_data):
        input_data = format_data(input_data)
        super(PNN, self).predict(input_data)

        if self.classes is None:
            raise ValueError("Train network before predict data")

        input_data_size = input_data.shape[1]
        train_data_size = self.input_train.shape[1]

        if input_data_size != train_data_size:
            raise ValueError("Input data must contains {0} features, got "
                             "{1}".format(train_data_size, input_data_size))

        class_ratios = self.class_ratios
        pdf_outputs = pdf_between_data(self.input_train, input_data,
                                       self.std)
        return dot(
            self.row_comb_matrix, pdf_outputs
        ) / class_ratios.reshape((class_ratios.size, 1))

    def predict(self, input_data):
        raw_output = self.predict_raw(input_data)
        return self.classes[raw_output.argmax(axis=0)]



import collections

import six
import theano.tensor as T
from theano.tensor.signal import pool

from neupy.core.properties import TypedListProperty, Property, ChoiceProperty
from .base import BaseLayer, ParameterBasedLayer


__all__ = ('Convolution', 'MaxPooling', 'AveragePooling')


class StrideProperty(TypedListProperty):
    """ Stride property.

    Parameters
    ----------
    {BaseProperty.default}
    {BaseProperty.required}
    """
    expected_type = (list, tuple, set, int)

    def __init__(self, *args, **kwargs):
        kwargs['element_type'] = int
        super(StrideProperty, self).__init__(*args, **kwargs)

    def __set__(self, instance, value):
        if isinstance(value, collections.Iterable) and len(value) == 1:
            value = value[0]

        if isinstance(value, int):
            value = (value, 1)

        super(StrideProperty, self).__set__(instance, value)

    def validate(self, value):
        super(StrideProperty, self).validate(value)
        if len(value) > 2:
            raise ValueError("Stide can have only one or two elements "
                             "in the list. Got {}".format(len(value)))


class BorderModeProperty(Property):
    """ Border mode property identifies border for the
    convolution operation.
    """
    expected_type = (six.string_types, int, tuple)
    valid_string_choices = ('valid', 'full', 'half')

    def validate(self, value):
        super(BorderModeProperty, self).validate(value)

        if isinstance(value, tuple):
            if len(value) != 2:
                raise ValueError(
                    "Border mode property suppose to get a tuple that "
                    "contains two elements, got {} elements"
                    "".format(len(value))
                )

        is_invalid_string = (
            isinstance(value, six.string_types) and
            value not in self.valid_string_choices
        )
        if is_invalid_string:
            valid_choices = ', '.join(self.valid_string_choices)
            raise ValueError("`{}` is invalid string value. Available: {}"
                             "".format(value, valid_choices))


class Convolution(ParameterBasedLayer):
    """ Convolutional layer.

    Parameters
    ----------
    size : tuple of integers
        Filter shape.
    border_mode : {{'valid', 'full', 'half'}} or int or tuple with 2 int
        Convolution border mode. Check Theano's `nnet.conv2d` doc.
    stride_size : tuple with 1 or 2 integers or integer.
        Stride size.
    """
    size = TypedListProperty(required=True, element_type=int)
    border_mode = BorderModeProperty(default='valid')
    stride_size = StrideProperty(default=(1, 1))

    def weight_shape(self):
        return self.size

    def bias_shape(self):
        return self.size[:1]

    def output(self, input_value):
        bias = T.reshape(self.bias, (1, -1, 1, 1))
        output = T.nnet.conv2d(input_value, self.weight,
                               border_mode=self.border_mode,
                               subsample=self.stride_size)
        return output + bias


class BasePooling(BaseLayer):
    """ Base class for the pooling layers.

    Parameters
    ----------
    size : tuple with 2 integers
        Factor by which to downscale (vertical, horizontal).
        (2,2) will halve the image in each dimension.
    stride_size :
        Stride size, which is the number of shifts over
        rows/cols to get the next pool region. If stride_size is
        None, it is considered equal to ds (no overlap on
        pooling regions).
    padding : tuple of two ints
        (pad_h, pad_w), pad zeros to extend beyond four borders of
        the images, pad_h is the size of the top and bottom margins,
        and pad_w is the size of the left and right margins.
    """
    size = TypedListProperty(required=True, element_type=int)
    stride_size = StrideProperty(default=None)
    padding = TypedListProperty(default=(0, 0), element_type=int, n_elements=2)

    def __init__(self, size, **options):
        options['size'] = size
        super(BasePooling, self).__init__(**options)

    def __repr__(self):
        return '{name}({size})'.format(name=self.__class__.__name__,
                                       size=self.size)


class MaxPooling(BasePooling):
    """ Maximum pooling layer.

    Parameters
    ----------
    {BasePooling.size}
    {BasePooling.stride_size}
    """
    def output(self, input_value):
        return pool.pool_2d(input_value, ds=self.size, mode='max',
                            ignore_border=True, st=self.stride_size,
                            padding=self.padding)


class AveragePooling(BasePooling):
    """ Average pooling layer.

    Parameters
    ----------
    mode : {{'include_padding', 'exclude_padding'}}
        Gives you the choice to include or exclude padding.
        Defaults to ``include_padding``.
    {BasePooling.size}
    {BasePooling.stride_size}
    """
    mode = ChoiceProperty(
        default='include_padding',
        choices={
            'include_padding': 'average_inc_pad',
            'exclude_padding': 'average_exc_pad'
        }
    )

    def output(self, input_value):
        return pool.pool_2d(input_value, ds=self.size, mode=self.mode,
                            ignore_border=True, st=self.stride_size,
                            padding=self.padding)



import math
import time
from abc import ABCMeta, abstractmethod
from collections import deque

import six
import numpy as np

from neupy.helpers.table import format_time


__all__ = ('SummaryTable', 'InlineSummary')


class BaseSummary(six.with_metaclass(ABCMeta)):
    @abstractmethod
    def show_last(self, network):
        pass

    @abstractmethod
    def finish(self):
        pass


class SummaryTable(BaseSummary):
    """ Class that shows network's training in the formatted
    table.

    Parameters
    ----------
    network : BaseNetwork
        Network instance.
    table_builder : TableBuilder
        Pre-defined table builder with specified table
        structure.
    delay_limit : float
    delay_history_length : int
    """
    def __init__(self, network, table_builder, delay_limit=1.,
                 delay_history_length=10):

        self.network = network
        self.table_builder = table_builder
        self.delay_limit = delay_limit
        self.delay_history_length = delay_history_length

        self.prev_summary_time = None
        self.terminal_output_delays = deque(maxlen=delay_history_length)

        table_builder.start()

    def show_last(self):
        network = self.network
        table_builder = self.table_builder
        terminal_output_delays = self.terminal_output_delays

        now = time.time()

        if self.prev_summary_time is not None:
            time_delta = now - self.prev_summary_time
            terminal_output_delays.append(time_delta)

        table_builder.row([
            network.last_epoch,
            network.errors.last() or '-',
            network.validation_errors.last() or '-',
            network.training.epoch_time,
        ])
        self.prev_summary_time = now

        if len(terminal_output_delays) == self.delay_history_length:
            self.prev_summary_time = None
            average_delay = np.mean(terminal_output_delays)

            if average_delay < self.delay_limit:
                show_epoch = int(
                    network.training.show_epoch *
                    math.ceil(self.delay_limit / average_delay)
                )
                table_builder.message("Too many outputs in the terminal. "
                                      "Set up logging after each {} epochs"
                                      "".format(show_epoch))

                terminal_output_delays.clear()
                network.training.show_epoch = show_epoch

    def finish(self):
        self.table_builder.finish()


class InlineSummary(BaseSummary):
    def __init__(self, network):
        self.network = network

    def show_last(self):
        network = self.network
        logs = network.logs

        train_error = network.errors.last()
        validation_error = network.validation_errors.last()
        epoch_training_time = format_time(network.training.epoch_time)

        if validation_error is not None:
            logs.write(
                "epoch #{}, train err: {:.6f}, valid err: {:.6f}, time: {}"
                "".format(network.last_epoch, train_error, validation_error,
                          epoch_training_time)
            )
        else:
            logs.write(
                "epoch #{}, train err: {:.6f}, time: {}"
                "".format(network.last_epoch, train_error,
                          epoch_training_time)
            )

    def finish(self):
        pass



import numpy as np
import theano.tensor as T

from neupy.core.properties import ProperFractionProperty, TypedListProperty
from .base import BaseLayer


__all__ = ('Dropout', 'Reshape')


class Dropout(BaseLayer):
    """ Dropout layer

    Parameters
    ----------
    proba : float
        Fraction of the input units to drop. Value needs to be
        between 0 and 1.
    """
    proba = ProperFractionProperty(required=True)

    def __init__(self, proba, **options):
        options['proba'] = proba
        super(Dropout, self).__init__(**options)

    @property
    def size(self):
        return self.relate_to_layer.size

    def output(self, input_value):
        # Use NumPy seed to make Theano code easely reproducible
        max_possible_seed = 4e9
        seed = np.random.randint(max_possible_seed)
        theano_random = T.shared_randomstreams.RandomStreams(seed)

        proba = (1.0 - self.proba)
        mask = theano_random.binomial(n=1, p=proba,
                                      size=input_value.shape,
                                      dtype=input_value.dtype)
        return (mask * input_value) / proba

    def __repr__(self):
        return "{name}(proba={proba})".format(
            name=self.__class__.__name__,
            proba=self.proba
        )


class Reshape(BaseLayer):
    """ Gives a new shape to an input value without changing
    its data.

    Parameters
    ----------
    shape : tuple or list
        New feature shape. ``None`` value means that feature
        will be flatten in 1D vector. If you need to get the
        output feature with more that 2 dimensions then you can
        set up new feature shape using tuples. Defaults to ``None``.
    """
    shape = TypedListProperty()

    def __init__(self, shape=None, **options):
        if shape is not None:
            options['shape'] = shape
        super(Reshape, self).__init__(**options)

    def output(self, input_value):
        """ Reshape the feature space for the input value.

        Parameters
        ----------
        input_value : array-like or Theano variable
        """
        new_feature_shape = self.shape
        input_shape = input_value.shape[0]

        if new_feature_shape is None:
            output_shape = input_value.shape[1:]
            new_feature_shape = T.prod(output_shape)
            output_shape = (input_shape, new_feature_shape)
        else:
            output_shape = (input_shape,) + new_feature_shape

        return T.reshape(input_value, output_shape)



from __future__ import division

import math

import six
import theano
import theano.tensor as T
import numpy as np

from neupy.core.properties import Property, BoundedProperty
from neupy.network import ConstructableNetwork
from . import addon_types


__all__ = ('GradientDescent', 'MinibatchGradientDescent')


class GradientDescent(ConstructableNetwork):
    """ GradientDescent algorithm.

    Parameters
    ----------
    addons : list or None
        The list of addon algortihms. ``None`` by default.
        If this option is not empty it will generate new class which
        will inherit all from this list. Support two types of
        addon algorithms: weight update and step update.
    {ConstructableNetwork.connection}
    {ConstructableNetwork.error}
    {BaseNetwork.step}
    {BaseNetwork.show_epoch}
    {BaseNetwork.shuffle_data}
    {BaseNetwork.epoch_end_signal}
    {BaseNetwork.train_end_signal}
    {Verbose.verbose}

    Attributes
    ----------
    {BaseNetwork.errors}
    {BaseNetwork.train_errors}
    {BaseNetwork.validation_errors}
    {BaseNetwork.last_epoch}

    Methods
    -------
    {BaseSkeleton.predict}
    {SupervisedLearning.train}
    {BaseSkeleton.fit}

    Examples
    --------
    >>> import numpy as np
    >>> from neupy.algorithms import GradientDescent
    >>>
    >>> x_train = np.array([[1, 2], [3, 4]])
    >>> y_train = np.array([[1], [0]])
    >>>
    >>> bpnet = GradientDescent((2, 3, 1), verbose=False, step=0.1)
    >>> bpnet.train(x_train, y_train)
    """
    supported_addon_types = addon_types.keys()

    addons = Property(default=None, expected_type=list)

    # TODO: The None parameters that get useful only in
    # case of dill.load operation don't look good.
    # I should find a better way to solve this problem
    # The same I need to do with `__init__` method
    def __new__(cls, connection=None, options=None, floatX=None, **kwargs):
        # Argument `options` is a simple hack for the `__reduce__` method.
        # `__reduce__` can't retore class with keyword arguments and
        # it will put them as `dict` argument in the `options` and method
        # will translate it to kwargs. The same hack is in the
        # `__init__` method.

        if options is None:
            options = kwargs

        if floatX is not None:
            theano.config.floatX = floatX

        addons = options.get('addons')

        if not addons:
            cls.main_class = cls
            return super(GradientDescent, cls).__new__(cls)

        identified_types = []
        for addon_class in addons:
            opt_class_type = getattr(addon_class, 'addon_type',  None)

            if opt_class_type not in cls.supported_addon_types:
                opt_class_name = addon_class.__name__
                supported_opts = ', '.join(addon_types.values())
                raise ValueError(
                    "Invalid add-on class `{}`. Class supports only "
                    "{}".format(opt_class_name, supported_opts)
                )

            if opt_class_type in identified_types:
                raise ValueError(
                    "There can be only one add-on class with "
                    "type `{}`".format(addon_types[opt_class_type])
                )

            identified_types.append(opt_class_type)

        new_class_name = (
            cls.__name__ +
            ''.join(class_.__name__ for class_ in addons)
        )
        mro_classes = tuple(list(addons) + [cls])
        new_class = type(new_class_name, mro_classes, {})
        new_class.main_class = cls

        return super(GradientDescent, new_class).__new__(new_class)

    def __init__(self, connection, options=None, floatX=None, **kwargs):
        if options is None:
            options = kwargs
        super(GradientDescent, self).__init__(connection, **options)

    def init_param_updates(self, layer, parameter):
        step = self.variables.step
        gradient = T.grad(self.variables.error_func, wrt=parameter)
        return [(parameter, parameter - step * gradient)]

    def class_name(self):
        return self.main_class.__name__

    def get_params(self, deep=False, with_connection=True):
        params = super(GradientDescent, self).get_params()
        if with_connection:
            params['connection'] = self.connection
        return params

    def __reduce__(self):
        parameters = self.get_params(with_connection=False)
        floatX = theano.config.floatX
        args = (self.connection, parameters, floatX)
        return (self.main_class, args)


class BatchSizeProperty(BoundedProperty):
    """ Batch size property

    Parameters
    ----------
    {BoundedProperty.maxval}
    {BaseProperty.default}
    {BaseProperty.required}
    """
    expected_type = (type(None), int)
    fullbatch_identifiers = [None, -1, 'all', '*', 'full']

    def __init__(self, *args, **kwargs):
        super(BatchSizeProperty, self).__init__(minval=1, *args, **kwargs)

    def __set__(self, instance, value):
        if isinstance(value, six.string_types):
            value = value.lower()

        if value in self.fullbatch_identifiers:
            value = None

        super(BatchSizeProperty, self).__set__(instance, value)

    def validate(self, value):
        if value is not None:
            super(BatchSizeProperty, self).validate(value)


def iter_batches(n_samples, batch_size):
    """ Iterates batch slices.

    Parameters
    ----------
    n_samples : int
        Number of samples. Number should be greater than 0.
    batch_size : int
        Batch size. Number should be greater than 0.

    Yields
    ------
    object
        Batch slices.
    """
    n_batches = int(math.ceil(n_samples / batch_size))

    for batch_index in range(n_batches):
        yield slice(
            batch_index * batch_size,
            (batch_index + 1) * batch_size
        )


def cannot_divide_into_batches(data, batch_size):
    """ Checkes whether data can be divided into at least
    two batches.

    Parameters
    ----------
    data : array-like
    batch_size : int or None

    Returns
    -------
    bool
    """
    n_samples = len(data)
    return batch_size is None or n_samples <= batch_size


def apply_batches(function, arguments, batch_size, logger, description='',
                  show_progressbar=False, use_error_output=True):
    """ Apply batches to a specified function.

    Parameters
    ----------
    function : func
        Function that accepts one or more positional arguments.
        Each of them should be an array-like variable that
        have exactly the same number of rows.
    arguments : tuple, list
        The arguemnts that will be provided to the function specified
        in the ``function`` argument.
    batch_size : int
        Batch size.
    logger : TerminalLogger instance
    description : str
        Short description that will be displayed near the progressbar
        in verbose mode. Defaults to ``''`` (empty string).
    show_progressbar : bool
        ``True`` mean that function will show progressbar in the
        terminal. Defaults to ``False``.

    Returns
    -------
    list
        List of function outputs.
    """
    if not arguments:
        raise ValueError("The arguments parameter should have at "
                         "least one element.")

    samples = arguments[0]
    n_samples = len(samples)
    batch_iterator = iter_batches(n_samples, batch_size)

    if show_progressbar:
        batch_iterator = logger.progressbar(
            list(batch_iterator),
            desc=description,
            file=logger.stdout
        )

    output = None
    outputs = []
    for batch in batch_iterator:
        if show_progressbar and logger.enable:
            batch = batch_iterator.send(output if use_error_output else None)

        sliced_arguments = [argument[batch] for argument in arguments]
        output = function(*sliced_arguments)

        outputs.append(output)

    return outputs


def average_batch_errors(errors, n_samples, batch_size):
    """ Computes average error per sample.

    Parameters
    ----------
    errors : list
        List of errors where each element is a average error
        per batch.
    n_samples : int
        Number of samples in the dataset.
    batch_size : int
        Batch size.

    Returns
    -------
    float
        Average error per sample.
    """
    n_samples_in_final_batch = n_samples % batch_size

    if n_samples_in_final_batch == 0:
        return np.mean(errors)

    all_errors_without_last = errors[:-1]
    last_error = errors[-1]

    total_error = (
        sum(all_errors_without_last) * batch_size +
        last_error * n_samples_in_final_batch
    )
    average_error = (total_error / n_samples)
    return average_error


class MinibatchGradientDescent(GradientDescent):
    """ Mini-batch Gradient Descent algorithm.

    Parameters
    ----------
    batch_size : int or {{None, -1, 'all', '*', 'full'}}
        Set up batch size for learning process. To set up batch size equal to
        sample size value should be equal to one of the values listed above.
        Defaults to ``100``.
    {GradientDescent.addons}
    {ConstructableNetwork.connection}
    {ConstructableNetwork.error}
    {BaseNetwork.step}
    {BaseNetwork.show_epoch}
    {BaseNetwork.shuffle_data}
    {BaseNetwork.epoch_end_signal}
    {BaseNetwork.train_end_signal}
    {Verbose.verbose}

    Attributes
    ----------
    {BaseNetwork.errors}
    {BaseNetwork.train_errors}
    {BaseNetwork.validation_errors}
    {BaseNetwork.last_epoch}

    Methods
    -------
    {BaseSkeleton.predict}
    {SupervisedLearning.train}
    {BaseSkeleton.fit}

    Examples
    --------
    >>> import numpy as np
    >>> from neupy import algorithms
    >>>
    >>> x_train = np.array([[1, 2], [3, 4]])
    >>> y_train = np.array([[1], [0]])
    >>>
    >>> mgdnet = algorithms.MinibatchGradientDescent(
    ...     (2, 3, 1), batch_size=1
    ... )
    >>> mgdnet.train(x_train, y_train)

    See Also
    --------
    :network:`GradientDescent` : GradientDescent algorithm.
    """
    batch_size = BatchSizeProperty(default=100)

    def train_epoch(self, input_train, target_train):
        """ Train one epoch.

        Parameters
        ----------
        input_train : array-like
            Training input array.
        target_train : array-like
            Training target array.

        Returns
        -------
        float
            Training error.
        """
        train_epoch = self.methods.train_epoch

        if cannot_divide_into_batches(input_train, self.batch_size):
            return train_epoch(input_train, target_train)

        show_progressbar = (self.training and self.training.show_epoch == 1)
        errors = apply_batches(
            function=train_epoch,
            arguments=(input_train, target_train),
            batch_size=self.batch_size,

            description='Training batches',
            show_progressbar=show_progressbar,
            logger=self.logs,
            use_error_output=True,
        )
        return average_batch_errors(
            errors,
            n_samples=len(input_train),
            batch_size=self.batch_size,
        )

    def prediction_error(self, input_data, target_data):
        """ Check the prediction error for the specified input samples
        and their targets.

        Parameters
        ----------
        input_data : array-like
        target_data : array-like

        Returns
        -------
        float
            Prediction error.
        """
        input_data = self.format_input_data(input_data)
        target_data = self.format_target_data(target_data)

        prediction_error = self.methods.prediction_error

        if cannot_divide_into_batches(input_data, self.batch_size):
            return prediction_error(input_data, target_data)

        show_progressbar = (self.training and self.training.show_epoch == 1)
        errors = apply_batches(
            function=prediction_error,
            arguments=(input_data, target_data),
            batch_size=self.batch_size,

            description='Validation batches',
            show_progressbar=show_progressbar,
            logger=self.logs,
            use_error_output=True,
        )
        return average_batch_errors(
            errors,
            n_samples=len(input_data),
            batch_size=self.batch_size,
        )

    def predict_raw(self, input_data):
        """ Makes a raw prediction.

        Parameters
        ----------
        input_data : array-like

        Returns
        -------
        array-like
        """
        input_data = self.format_input_data(input_data)
        predict_raw = self.methods.predict_raw

        if cannot_divide_into_batches(input_data, self.batch_size):
            return predict_raw(input_data)

        outputs = apply_batches(
            function=predict_raw,
            arguments=(input_data,),
            batch_size=self.batch_size,

            description='Prediction batches',
            show_progressbar=True,
            logger=self.logs,
            use_error_output=False,
        )

        return np.concatenate(outputs, axis=0)



import numpy as np

from neupy.utils import preformat_value
from neupy.core.docs import SharedDocs


__all__ = ('BaseProperty', 'Property', 'ArrayProperty', 'BoundedProperty',
           'ProperFractionProperty', 'NumberProperty', 'IntProperty',
           'TypedListProperty', 'ChoiceProperty')


class BaseProperty(SharedDocs):
    """ Base class for properties.

    Parameters
    ----------
    default : object
        Default property value. Defaults to ``None``.
    required : bool
        If parameter equal to ``True`` and value undefined after
        initialization class then it will cause an error.
        Defaults to ``False``.
    """
    expected_type = object

    def __init__(self, default=None, required=False):
        self.name = None
        self.default = default
        self.required = required

    def __set__(self, instance, value):
        if not isinstance(value, self.expected_type):
            availabe_types = self.expected_type
            if isinstance(availabe_types, (list, tuple)):
                availabe_types = ', '.join(t.__name__ for t in availabe_types)
            else:
                availabe_types = availabe_types.__name__

            raise TypeError(
                "Invalid data type `{0}` for `{1}` property. Expected "
                "types: {2}".format(
                    value.__class__.__name__,
                    self.name,
                    availabe_types
                )
            )

        self.validate(value)
        instance.__dict__[self.name] = value

    def __get__(self, instance, value):
        if instance is None:
            return

        if self.default is not None and self.name not in instance.__dict__:
            self.__set__(instance, self.default)

        return instance.__dict__.get(self.name, None)

    def __delete__(self, instance):
        name = self.name

        if name in instance.__dict__:
            del instance.__dict__[name]

        if name in instance.options:
            del instance.options[name]

    def __str__(self):
        return str(preformat_value(self.default))

    def __repr__(self):
        return self.__str__()

    def validate(self, value):
        pass


class Property(BaseProperty):
    """ Simple and flexible class that helps indetify properties with
    specified type.

    Parameters
    ----------
    expected_type : object
        Valid data type.
    {BaseProperty.default}
    {BaseProperty.required}
    """
    def __init__(self, expected_type=object, *args, **kwargs):
        self.expected_type = expected_type
        super(Property, self).__init__(*args, **kwargs)


class ArrayProperty(BaseProperty):
    """ Numpy array or matrix property.

    Parameters
    ----------
    {BaseProperty.default}
    {BaseProperty.required}
    """
    expected_type = (np.ndarray, np.matrix)


class TypedListProperty(BaseProperty):
    """ List property that contains specified element types.

    Parameters
    ----------
    n_elements : int
        Indentify fixed number of elements in list. ``None`` value mean
        that list can contains any number of elements. Defaults to ``None``.
    element_type : object or tuple
        There are could be defined valid list elementy type or a bunch
        of them as tuple.
    {BaseProperty.default}
    {BaseProperty.required}
    """
    expected_type = (list, tuple)

    def __init__(self, n_elements=None, element_type=int, *args, **kwargs):
        self.n_elements = n_elements
        self.element_type = element_type
        super(TypedListProperty, self).__init__(*args, **kwargs)

    def validate(self, value):
        super(TypedListProperty, self).validate(value)

        if self.n_elements is not None and len(value) != self.n_elements:
            raise ValueError("Expected list with {} variables"
                             "".format(self.n_elements))

        if not all(isinstance(v, self.element_type) for v in value):
            if isinstance(self.element_type, tuple):
                type_names = (type_.__name__ for type_ in self.element_type)
                element_type_name = ', '.join(type_names)
            else:
                element_type_name = self.element_type.__name__

            raise TypeError("Valid list element types for `{}` are: {}"
                            "".format(self.name, element_type_name))


class ChoiceProperty(BaseProperty):
    """ Property that can have discrete number of properties.

    Parameters
    ----------
    choices : list, tuple or dict
        Identify all posible choices. Dictionary choices ties values
        with some names that can help easily chang options between
        some specific object like functions. List or tuple choices
        do the same as dictionary, but they are useful in case when
        keys and values should be the same.
    {BaseProperty.default}
    {BaseProperty.required}
    """
    choices = {}

    def __init__(self, choices, *args, **kwargs):
        super(ChoiceProperty, self).__init__(*args, **kwargs)
        self.choices = choices

        if isinstance(choices, (list, tuple, set)):
            self.choices = dict(zip(choices, choices))

        if not isinstance(self.choices, dict):
            class_name = self.choices.__class__.__name__
            raise ValueError("Choice properties can be only a `dict`, got "
                             "`{0}`".format(class_name))

        if not self.choices:
            raise ValueError("Must be at least one choice in property "
                             "`{0}`".format(self.name))

    def __set__(self, instance, value):
        if value in self.choices:
            return super(ChoiceProperty, self).__set__(instance, value)

        possible_choices = ", ".join(self.choices.keys())
        raise ValueError(
            "Wrong value `{0}` for property `{1}`. Available values: "
            "{2}".format(value, self.name, possible_choices)
        )

    def __get__(self, instance, value):
        if instance is not None:
            choice_key = super(ChoiceProperty, self).__get__(instance, value)
            return self.choices[choice_key]


class BoundedProperty(BaseProperty):
    """ Number property that have specified numerical bounds.

    Parameters
    ----------
    minval : float
        Minimum possible value for the property.
    maxval : float
        Maximum possible value for the property.
    {BaseProperty.default}
    {BaseProperty.required}
    """

    def __init__(self, minval=-np.inf, maxval=np.inf, *args, **kwargs):
        self.minval = minval
        self.maxval = maxval
        super(BoundedProperty, self).__init__(*args, **kwargs)

    def validate(self, value):
        super(BoundedProperty, self).validate(value)

        if not (self.minval <= value <= self.maxval):
            raise ValueError("Value `{}` should be between {} and {}"
                             "".format(self.name, self.minval, self.maxval))


class ProperFractionProperty(BoundedProperty):
    """ Proper fraction property. Identify all possible numbers
    between zero and one.

    Parameters
    ----------
    {BaseProperty.default}
    {BaseProperty.required}
    """
    expected_type = (float, int)

    def __init__(self, *args, **kwargs):
        super(ProperFractionProperty, self).__init__(minval=0, maxval=1,
                                                     *args, **kwargs)


class NumberProperty(BoundedProperty):
    """ Float or integer number property.

    Parameters
    ----------
    {BoundedProperty.minval}
    {BoundedProperty.maxval}
    {BaseProperty.default}
    {BaseProperty.required}
    """
    expected_type = (float, int)


class IntProperty(BoundedProperty):
    """ Integer property.

    Parameters
    ----------
    {BoundedProperty.minval}
    {BoundedProperty.maxval}
    {BaseProperty.default}
    {BaseProperty.required}
    """
    expected_type = (int, np.integer)



from neupy.utils import format_data


__all__ = ('SupervisedLearning', 'UnsupervisedLearning', 'LazyLearning')


class SupervisedLearning(object):
    """ Mixin for Supervised Neural Network algorithms.

    Methods
    -------
    train(input_train, target_train, input_test=None, target_test=None,\
    epochs=100, epsilon=None)
        Trains network. You can control network training procedure
        iterations with the number of epochs or converge value epsilon.
        Also you can specify ``input_test`` and ``target_test`` and control
        your validation data error on each iteration.
    """
    def train(self, input_train, target_train, input_test=None,
              target_test=None, epochs=100, epsilon=None,
              summary_type='table'):

        is_test_data_partialy_missed = (
            (input_test is None and target_test is not None) or
            (input_test is not None and target_test is None)
        )

        if is_test_data_partialy_missed:
            raise ValueError("Input and target test samples missed. "
                             "They must be defined both or none of them.")

        input_train = format_data(input_train)
        target_train = format_data(target_train)

        if input_test is not None:
            input_test = format_data(input_test)

        if target_test is not None:
            target_test = format_data(target_test)

        return super(SupervisedLearning, self).train(
            input_train=input_train, target_train=target_train,
            input_test=input_test, target_test=target_test,
            epochs=epochs, epsilon=epsilon,
            summary_type=summary_type
        )


class UnsupervisedLearning(object):
    """ Mixin for Unsupervised Neural Network algorithms.

    Methods
    -------
    train(input_train, epsilon=1e-5, epochs=100)
        Trains network until it converge. Parameter ``epochs`` control
        maximum number of iterations, just to make sure that network will
        stop training procedure if it can't converge.
    """
    def train(self, input_train, epochs=100, epsilon=None,
              summary_type='table'):

        input_train = format_data(input_train, is_feature1d=True)
        return super(UnsupervisedLearning, self).train(
            input_train=input_train, target_train=None,
            input_test=None, target_test=None,
            epochs=epochs, epsilon=epsilon,
            summary_type=summary_type
        )


class LazyLearning(object):
    """ Mixin for lazy learning Neural Network algorithms.

    Methods
    -------
    train(input_train, target_train, copy=True)
        Network just stores all the information about the data and use it for \
        the prediction. Parameter ``copy`` copy input data before store it \
        inside the network.
    """
    def __init__(self, *args, **kwargs):
        self.input_train = None
        self.target_train = None
        super(LazyLearning, self).__init__(*args, **kwargs)

    def init_properties(self):
        del self.shuffle_data
        del self.step
        del self.show_epoch
        del self.train_end_signal
        del self.epoch_end_signal

    def train(self, input_train, target_train):
        self.input_train = input_train
        self.target_train = target_train

        if input_train.shape[0] != target_train.shape[0]:
            raise ValueError("Input data size should be the same as "
                             "target data size")

    def train_epoch(self, *args, **kwargs):
        raise AttributeError("This network doesn't have train epochs")

    def predict(self, input_data):
        if self.input_train is None:
            raise ValueError("Train neural network before make prediction")



from __future__ import print_function

import os
import sys
import importlib
from contextlib import contextmanager

from neupy.core.config import Configurable
from neupy.core.properties import BaseProperty
from neupy.helpers import progressbar
from . import terminal


__all__ = ('Verbose',)


def terminal_echo(enabled, file_descriptor=sys.stdin):
    """ Enable/disable echo in the terminal.

    Parameters
    ----------
    enabled : bool
        The `False` value means that you willn't be able to make
        input in terminal.
    file_descriptor : object
        File descriptor that you would like to enable or disable.
        Defaults to ``sys.stdin``.
    """
    is_windows = (os.name == 'nt')

    if not is_windows:
        termios = importlib.import_module('termios')

        try:
            attributes = termios.tcgetattr(file_descriptor)
            lflag = attributes[3]

            if enabled:
                lflag |= termios.ECHO
            else:
                lflag &= ~termios.ECHO

            attributes[3] = lflag
            termios.tcsetattr(file_descriptor, termios.TCSANOW, attributes)

        except termios.error:
            pass


class TerminalLogger(object):
    """ Customized logging class that replace standard logging
    functionality.

    Attributes
    ----------
    enable : bool
        Enable/disable logging output. Defaults to ``True``.
    template : str
        Terminal output message template. Defaults
        to ``"[{name}] {text}"``.
    stdout : object
        Writes output in terminal. Defaults to ``sys.stdout``.
    """

    colors = {
        'gray': terminal.gray,
        'green': terminal.green,
        'red': terminal.red,
        'white': terminal.white,
    }

    def __init__(self):
        self.enable = True
        self.template = "[{tag}] {text}"
        self.stdout = sys.stdout

    def write(self, text):
        """ Method writes text in terminal if logging is enable.

        Parameters
        ----------
        text : str
        """
        if self.enable:
            self.stdout.write(str(text) + '\n')

    def newline(self):
        """ Just writes an empty line.
        """
        self.write('\r')

    def message(self, tag, text, color='green'):
        """ Methods writes message in terminal using specific template.
        Each row should have tag and text. Tag identifies message
        category and text information reletad to this category.

        Parameters
        ----------
        name : str
        text : str
        color : {{'green', 'gray', 'red', 'white'}}
            Property that color text defined as ``tag`` parameter.
            Defaults to ``green``.
        """

        if color not in self.colors:
            available_colors = ', '.join(self.colors.keys())
            raise ValueError("Invalid color `{}`. Available colors: {}"
                             "".format(color, available_colors))

        colorizer = self.colors[color]
        formated_tag = colorizer(tag.upper())
        message = self.template.format(tag=formated_tag, text=text)
        self.write(message)

    def title(self, text):
        """ Method write text as a title message. Text will be displayed
        using bold and underline text styles. Also there will be empty
        lines before and after the message.

        Parameters
        ----------
        text : str
        """
        bold_text = terminal.bold(text)
        message = "\n{text}\n".format(text=terminal.underline(bold_text))
        self.write(message)

    def error(self, text):
        """" Method writes messages that related to error type.
        Text will be displayed as message with ``tag`` parameter equal
        to ``'ERROR'``. Color will be red.

        Parameters
        ----------
        text : str
        """
        self.message('ERROR', text, color='red')

    def warning(self, text):
        """" Method writes messages that related to warning type.
        Text will be displayed as message with ``tag`` parameter equal
        to ``'WARN'``. Color will be red.

        Parameters
        ----------
        text : str
        """
        self.message('WARN', text, color='red')

    def progressbar(self, iterator, *args, **kwargs):
        """ Make progressbar for specific iteration if logging
        is enable.

        Parameters
        ----------
        iterator : iterable object
        *args
            Arguments for ``progressbar`` function.
        **kwargs
            Key defined arguments for ``progressbar`` function.
        """
        if self.enable:
            return progressbar(iterator, *args, **kwargs)
        return iterator

    @contextmanager
    def disable_user_input(self):
        """ Context manager helps ignore user input in
        terminal for UNIX.
        """

        if self.enable:
            try:
                terminal_echo(enabled=False)
                yield
            finally:
                terminal_echo(enabled=True)
        else:
            yield


class VerboseProperty(BaseProperty):
    """ Property that synchronize updates with ``enable`` attribute in
    logging instance.

    Parameters
    ----------
    {BaseProperty.default}
    {BaseProperty.required}
    """
    expected_type = bool

    def __set__(self, instance, value):
        instance.logs.enable = value
        return super(VerboseProperty, self).__set__(instance, value)


class Verbose(Configurable):
    """ Class that controls NeuPy logging.

    Parameters
    ----------
    verbose : bool
        Property controls verbose output interminal. ``True`` enables
        informative output in the terminal and ``False`` -
        disable it. Defaults to ``False``.

    Attributes
    ----------
    logs : TerminalLogger
        ``TerminalLogger`` instance.
    """
    verbose = VerboseProperty(default=False)

    def __init__(self, **options):
        self.logs = TerminalLogger()
        self.logs.enable = self.verbose
        super(Verbose, self).__init__(**options)



from math import log, ceil

from numpy import zeros, fill_diagonal, random, sign

from neupy.utils import format_data
from neupy.network.utils import step
from neupy.core.properties import Property
from .utils import bin2sign, hopfield_energy
from .base import DiscreteMemory


__all__ = ('DiscreteHopfieldNetwork',)


class DiscreteHopfieldNetwork(DiscreteMemory):
    """ Discrete Hopfield Network. Memory algorithm which works only with
    binary vectors.

    Parameters
    ----------
    {DiscreteMemory.mode}
    {DiscreteMemory.n_times}
    check_limit : bool
        Option enable a limit of patterns control for the network using
        logarithmically proportion rule. Defaults to ``True``.

    Methods
    -------
    energy(input_data)
        Compute Discrete Hopfiel Energy.
    train(input_data)
        Save input data pattern into the network memory.
    predict(input_data, n_times=None)
        Recover data from the memory using input pattern.
        For the prediction procedure you can control number of iterations.
        If you set up this value equal to ``None`` then the value would be
        equal to the value that you set up for the property with
        the same name - ``n_times``.

    Notes
    -----
    * Input and output vectors should contain only binary values.

    Examples
    --------
    >>> import numpy as np
    >>> from neupy import algorithms
    >>>
    >>> def draw_bin_image(image_matrix):
    ...     for row in image_matrix.tolist():
    ...         print('| ' + ' '.join(' *'[val] for val in row))
    ...
    >>> zero = np.matrix([
    ...     0, 1, 1, 1, 0,
    ...     1, 0, 0, 0, 1,
    ...     1, 0, 0, 0, 1,
    ...     1, 0, 0, 0, 1,
    ...     1, 0, 0, 0, 1,
    ...     0, 1, 1, 1, 0
    ... ])
    >>>
    >>> one = np.matrix([
    ...     0, 1, 1, 0, 0,
    ...     0, 0, 1, 0, 0,
    ...     0, 0, 1, 0, 0,
    ...     0, 0, 1, 0, 0,
    ...     0, 0, 1, 0, 0,
    ...     0, 0, 1, 0, 0
    ... ])
    >>>
    >>> two = np.matrix([
    ...     1, 1, 1, 0, 0,
    ...     0, 0, 0, 1, 0,
    ...     0, 0, 0, 1, 0,
    ...     0, 1, 1, 0, 0,
    ...     1, 0, 0, 0, 0,
    ...     1, 1, 1, 1, 1,
    ... ])
    >>>
    >>> half_zero = np.matrix([
    ...     0, 1, 1, 1, 0,
    ...     1, 0, 0, 0, 1,
    ...     1, 0, 0, 0, 1,
    ...     0, 0, 0, 0, 0,
    ...     0, 0, 0, 0, 0,
    ...     0, 0, 0, 0, 0,
    ... ])
    >>>
    >>> draw_bin_image(zero.reshape((6, 5)))
    |   * * *
    | *       *
    | *       *
    | *       *
    | *       *
    |   * * *
    >>> draw_bin_image(half_zero.reshape((6, 5)))
    |   * * *
    | *       *
    | *       *
    |
    |
    |
    >>> data = np.concatenate([zero, one, two], axis=0)
    >>>
    >>> dhnet = algorithms.DiscreteHopfieldNetwork()
    >>> dhnet.train(data)
    >>>
    >>> result = dhnet.predict(half_zero)
    >>> draw_bin_image(result.reshape((6, 5)))
    |   * * *
    | *       *
    | *       *
    | *       *
    | *       *
    |   * * *

    See Also
    --------
    :ref:`password-recovery`: Password recovery with Discrete Hopfield Network.
    :ref:`discrete-hopfield-network`: Discrete Hopfield Network tutorial.
    """

    check_limit = Property(default=True, expected_type=bool)

    def __init__(self, **options):
        super(DiscreteHopfieldNetwork, self).__init__(**options)
        self.n_remembered_data = 0

    def train(self, input_data):
        self.discrete_validation(input_data)

        input_data = bin2sign(input_data)
        input_data = format_data(input_data, is_feature1d=False)

        nrows, n_features = input_data.shape
        nrows_after_update = self.n_remembered_data + nrows

        if self.check_limit:
            memory_limit = ceil(n_features / (2 * log(n_features)))

            if nrows_after_update > memory_limit:
                raise ValueError("You can't memorize more than {0} "
                                 "samples".format(memory_limit))

        weight_shape = (n_features, n_features)

        if self.weight is None:
            self.weight = zeros(weight_shape, dtype=int)

        if self.weight.shape != weight_shape:
            raise ValueError("Invalid input shapes. Number of input "
                             "features must be equal to {} and {} output "
                             "features".format(*weight_shape))

        self.weight = input_data.T.dot(input_data)
        fill_diagonal(self.weight, zeros(len(self.weight)))
        self.n_remembered_data = nrows_after_update

    def predict(self, input_data, n_times=None):
        self.discrete_validation(input_data)
        input_data = format_data(bin2sign(input_data), is_feature1d=False)

        if self.mode == 'async':
            if n_times is None:
                n_times = self.n_times

            _, n_features = input_data.shape
            output_data = input_data

            for _ in range(n_times):
                position = random.randint(0, n_features - 1)
                raw_new_value = output_data.dot(self.weight[:, position])
                output_data[:, position] = sign(raw_new_value)
        else:
            output_data = input_data.dot(self.weight)

        return step(output_data).astype(int)

    def energy(self, input_data):
        self.discrete_validation(input_data)
        input_data = bin2sign(input_data)
        input_data = format_data(input_data, is_feature1d=False)
        nrows, n_features = input_data.shape

        if nrows == 1:
            return hopfield_energy(self.weight, input_data, input_data)

        output = zeros(nrows)
        for i, row in enumerate(input_data):
            output[i] = hopfield_energy(self.weight, row, row)

        return output



import theano
import theano.tensor as T
import numpy as np

from neupy.utils import asfloat
from neupy.core.properties import ProperFractionProperty, Property
from .base import MinibatchGradientDescent


__all__ = ('Momentum',)


class Momentum(MinibatchGradientDescent):
    """ Momentum algorithm for :network:`GradientDescent` optimization.

    Parameters
    ----------
    momentum : float
        Control previous gradient ratio. Defaults to ``0.9``.
    nesterov : bool
        Instead of classic momentum computes Nesterov momentum.
        Defaults to ``False``.
    {MinibatchGradientDescent.batch_size}
    {GradientDescent.addons}
    {ConstructableNetwork.connection}
    {ConstructableNetwork.error}
    {BaseNetwork.step}
    {BaseNetwork.show_epoch}
    {BaseNetwork.shuffle_data}
    {BaseNetwork.epoch_end_signal}
    {BaseNetwork.train_end_signal}
    {Verbose.verbose}

    Methods
    -------
    {BaseSkeleton.predict}
    {SupervisedLearning.train}
    {BaseSkeleton.fit}

    Examples
    --------
    Simple example

    >>> import numpy as np
    >>> from neupy import algorithms
    >>>
    >>> x_train = np.array([[1, 2], [3, 4]])
    >>> y_train = np.array([[1], [0]])
    >>>
    >>> mnet = algorithms.Momentum(
    ...     (2, 3, 1),
    ...     verbose=False
    ... )
    >>> mnet.train(x_train, y_train)

    See Also
    --------
    :network:`GradientDescent` : GradientDescent algorithm.
    """
    momentum = ProperFractionProperty(default=0.9)
    nesterov = Property(default=False, expected_type=bool)

    def init_layers(self):
        super(Momentum, self).init_layers()
        for layer in self.layers:
            for parameter in layer.parameters:
                parameter_shape = T.shape(parameter).eval()
                parameter.prev_param_delta = theano.shared(
                    name="prev_param_delta_" + parameter.name,
                    value=asfloat(np.zeros(parameter_shape)),
                )

    def init_param_updates(self, layer, parameter):
        step = self.variables.step
        gradient = T.grad(self.variables.error_func, wrt=parameter)

        prev_param_delta = parameter.prev_param_delta
        parameter_delta = self.momentum * prev_param_delta - step * gradient

        if self.nesterov:
            parameter_delta = self.momentum * parameter_delta - step * gradient

        return [
            (parameter, parameter + parameter_delta),
            (prev_param_delta, parameter_delta),
        ]



from __future__ import division, absolute_import

import time
import types
from itertools import groupby

import six
import numpy as np

from neupy.utils import preformat_value, AttributeKeyDict
from neupy.helpers import table
from neupy.core.base import BaseSkeleton
from neupy.core.properties import (BoundedProperty, NumberProperty,
                                   Property)
from .summary_info import SummaryTable, InlineSummary
from .utils import (iter_until_converge, shuffle, normalize_error,
                    StopNetworkTraining)


__all__ = ('BaseNetwork',)


def show_network_options(network, highlight_options=None):
    """ Display all available parameters options for Neural Network.

    Parameters
    ----------
    network : object
        Neural network instance.
    highlight_options : list
        List of enabled options. In that case all options from that
        list would be marked with a green color.
    """
    available_classes = [cls.__name__ for cls in network.__class__.__mro__]
    logs = network.logs

    if highlight_options is None:
        highlight_options = {}

    def group_by_class_name(value):
        _, option = value
        option_priority = -available_classes.index(option.class_name)
        return option_priority, option.class_name

    grouped_options = groupby(
        sorted(network.options.items(), key=group_by_class_name),
        group_by_class_name,
    )

    logs.title("Main information")
    logs.message("ALGORITHM", network.class_name())
    logs.newline()

    for (_, class_name), options in grouped_options:
        if not options:
            continue

        logs.write("{}:".format(class_name))
        for key, data in sorted(options):
            if key in highlight_options:
                msg_color = 'green'
                value = highlight_options[key]
            else:
                msg_color = 'gray'
                value = data.value

            formated_value = preformat_value(value)
            msg_text = "{} = {}".format(key, formated_value)
            logs.message("OPTION", msg_text, color=msg_color)

        logs.newline()


def logging_info_about_the_data(network, input_train, input_test):
    logs = network.logs
    n_train_samples = input_train.shape[0]
    train_feature_shape = input_train.shape[1:]

    logs.title("Start training")
    logs.message("TRAIN DATA",
                 "{} samples, feature shape: {}"
                 "".format(n_train_samples, train_feature_shape))

    if input_test is not None:
        n_test_samples = input_test.shape[0]
        test_feature_shape = input_test.shape[1:]

        logs.message("TEST DATA",
                     "{} samples, feature shape: {}"
                     "".format(n_test_samples, test_feature_shape))

        if train_feature_shape != test_feature_shape:
            raise ValueError("Train and test samples should have the "
                             "same feature shape.")


def logging_info_about_training(network, epochs, epsilon):
    logs = network.logs
    if epsilon is None:
        logs.message("TRAINING", "Total epochs: {}".format(epochs))
    else:
        logs.message("TRAINING", "Epsilon: {}, Max epochs: {}"
                                 "".format(epsilon, epochs))


def parse_show_epoch_property(network, n_epochs, epsilon=None):
    show_epoch = network.show_epoch

    if isinstance(show_epoch, int):
        return show_epoch

    if epsilon is not None and isinstance(show_epoch, six.string_types):
        network.logs.warning("Can't use `show_epoch` value in converging "
                             "mode. Set up `show_epoch` property equal to 1")
        return 1

    number_end_position = show_epoch.index('time')
    # Ignore grammar mistakes like `2 time`, this error could be
    # really annoying
    n_epochs_to_check = int(show_epoch[:number_end_position].strip())

    if n_epochs <= n_epochs_to_check:
        return 1

    return int(round(n_epochs / n_epochs_to_check))


def create_training_epochs_iterator(network, epochs, epsilon=None):
    if epsilon is not None:
        return iter_until_converge(network, epsilon, max_epochs=epochs)

    next_epoch = network.last_epoch + 1
    return range(next_epoch, next_epoch + epochs)


class ShowEpochProperty(BoundedProperty):
    """ Class helps validate specific syntax for `show_epoch`
    property from ``BaseNetwork`` class.

    Parameters
    ----------
    {BoundedProperty.minval}
    {BoundedProperty.maxval}
    {BaseProperty.default}
    {BaseProperty.required}
    """
    expected_type = tuple([int] + [six.string_types])

    def validate(self, value):
        if not isinstance(value, six.string_types):
            if value < 1:
                raise ValueError("Property `{}` value should be integer "
                                 "greater than zero or string. See the "
                                 "documentation for more information."
                                 "".format(self.name))
            return

        if 'time' not in value:
            raise ValueError("`{}` value has invalid string format."
                             "".format(self.name))

        valid_endings = ('times', 'time')
        number_end_position = value.index('time')
        number_part = value[:number_end_position].strip()

        if not value.endswith(valid_endings) or not number_part.isdigit():
            valid_endings_formated = ', '.join(valid_endings)
            raise ValueError(
                "Property `{}` in string format should be a positive number "
                "with one of those endings: {}. For example: `10 times`."
                "".format(self.name, valid_endings_formated)
            )

        if int(number_part) < 1:
            raise ValueError("Part that related to the number in `{}` "
                             "property should be an integer greater or "
                             "equal to one.".format(self.name))


def is_valid_error_value(value):
    """ Checks that error value has valid type.

    Parameters
    ----------
    value : object

    Returns
    -------
    bool
    """
    return value is not None and not np.all(np.isnan(value))


class ErrorHistoryList(list):
    """ Wrapper around the built-in list class that adds a few
    additional methods.
    """
    def last(self):
        """ Returns last element if list is not empty,
        ``None`` otherwise.
        """
        if self and is_valid_error_value(self[-1]):
            return normalize_error(self[-1])

    def previous(self):
        """ Returns last element if list is not empty,
        ``None`` otherwise.
        """
        if len(self) >= 2 and is_valid_error_value(self[-2]):
            return normalize_error(self[-2])

    def normalized(self):
        """ Normalize list that contains error outputs.

        Returns
        -------
        list
            Return the same list with normalized values if there
            where some problems.
        """
        if not self or isinstance(self[0], float):
            return self

        normalized_errors = map(normalize_error, self)
        return list(normalized_errors)


class BaseNetwork(BaseSkeleton):
    """ Base class for Neural Network algorithms.

    Parameters
    ----------
    step : float
        Learning rate, defaults to ``0.1``.
    show_epoch : int or str
        This property controls how often the network will display information
        about training. There are two main syntaxes for this property.
        You can describe it as positive integer number and it
        will describe how offen would you like to see summary output in
        terminal. For instance, number `100` mean that network will show you
        summary in 100, 200, 300 ... epochs. String value should be in a
        specific format. It should contain the number of times that the output
        will be displayed in the terminal. The second part is just
        a syntax word ``time`` or ``times`` just to make text readable.
        For instance, value ``'2 times'`` mean that the network will show
        output twice with approximately equal period of epochs and one
        additional output would be after the finall epoch.
        Defaults to ``1``.
    shuffle_data : bool
        If it's ``True`` class shuffles all your training data before
        training your network, defaults to ``True``.
    epoch_end_signal : function
        Calls this function when train epoch finishes.
    train_end_signal : function
        Calls this function when train process finishes.
    {Verbose.verbose}

    Attributes
    ----------
    errors : ErrorHistoryList
        Contains list of training errors. This object has the same
        properties as list and in addition there are three additional
        useful methods: `last`, `previous` and `normalized`.
    train_errors : ErrorHistoryList
        Alias to `errors` attribute.
    validation_errors : ErrorHistoryList
        The same as `errors` attribute, but it contains only validation
        errors.
    last_epoch : int
        Value equals to the last trained epoch. After initialization
        it is equal to ``0``.
    """
    step = NumberProperty(default=0.1, minval=0)

    show_epoch = ShowEpochProperty(minval=1, default=1)
    shuffle_data = Property(default=False, expected_type=bool)

    epoch_end_signal = Property(expected_type=types.FunctionType)
    train_end_signal = Property(expected_type=types.FunctionType)

    def __init__(self, *args, **options):
        self.errors = self.train_errors = ErrorHistoryList()
        self.validation_errors = ErrorHistoryList()
        self.training = AttributeKeyDict()
        self.last_epoch = 0

        super(BaseNetwork, self).__init__(*args, **options)
        self.init_properties()

        if self.verbose:
            show_network_options(self, highlight_options=options)

    def init_properties(self):
        """ Setup default values before populate the options.
        """

    def predict(self, input_data):
        """ Return prediction results for the input data. Output result
        includes post-processing step related to the final layer that
        transforms output to convenient format for end-use.

        Parameters
        ----------
        input_data : array-like

        Returns
        -------
        array-like
        """

    def on_epoch_start_update(self, epoch):
        """ Function would be trigger before run all training procedure
        related to the current epoch.

        Parameters
        ----------
        epoch : int
            Current epoch number.
        """
        self.last_epoch = epoch

    def train_epoch(self, input_train, target_train=None):
        raise NotImplementedError()

    def prediction_error(self, input_test, target_test):
        raise NotImplementedError()

    def train(self, input_train, target_train=None, input_test=None,
              target_test=None, epochs=100, epsilon=None,
              summary_type='table'):
        """ Method train neural network.

        Parameters
        ----------
        input_train : array-like
        target_train : array-like or Npne
        input_test : array-like or None
        target_test : array-like or None
        epochs : int
            Defaults to `100`.
        epsilon : float or None
            Defaults to ``None``.
        """

        show_epoch = self.show_epoch
        logs = self.logs
        training = self.training = AttributeKeyDict()

        if epochs <= 0:
            raise ValueError("Number of epochs needs to be greater than 0.")

        if epsilon is not None and epochs <= 2:
            raise ValueError("Network should train at teast 3 epochs before "
                             "check the difference between errors")

        if summary_type == 'table':
            logging_info_about_the_data(self, input_train, input_test)
            logging_info_about_training(self, epochs, epsilon)
            logs.newline()

            summary = SummaryTable(
                table_builder=table.TableBuilder(
                    table.Column(name="Epoch #"),
                    table.NumberColumn(name="Train err"),
                    table.NumberColumn(name="Valid err"),
                    table.TimeColumn(name="Time", width=10),
                    stdout=logs.write
                ),
                network=self,
                delay_limit=1.,
                delay_history_length=10,
            )

        elif summary_type == 'inline':
            summary = InlineSummary(network=self)

        else:
            raise ValueError("`{}` is unknown summary type"
                             "".format(summary_type))

        iterepochs = create_training_epochs_iterator(self, epochs, epsilon)
        show_epoch = parse_show_epoch_property(self, epochs, epsilon)
        training.show_epoch = show_epoch

        # Storring attributes and methods in local variables we prevent
        # useless __getattr__ call a lot of times in each loop.
        # This variables speed up loop in case on huge amount of
        # iterations.
        training_errors = self.errors
        validation_errors = self.validation_errors
        shuffle_data = self.shuffle_data

        train_epoch = self.train_epoch
        epoch_end_signal = self.epoch_end_signal
        train_end_signal = self.train_end_signal
        on_epoch_start_update = self.on_epoch_start_update

        is_first_iteration = True
        can_compute_validation_error = (input_test is not None)
        last_epoch_shown = 0

        with logs.disable_user_input():
            for epoch in iterepochs:
                validation_error = np.nan
                epoch_start_time = time.time()
                on_epoch_start_update(epoch)

                if shuffle_data:
                    input_train, target_train = shuffle(input_train,
                                                        target_train)
                try:
                    train_error = train_epoch(input_train, target_train)

                    if can_compute_validation_error:
                        validation_error = self.prediction_error(input_test,
                                                                 target_test)

                    training_errors.append(train_error)
                    validation_errors.append(validation_error)

                    epoch_finish_time = time.time()
                    training.epoch_time = epoch_finish_time - epoch_start_time

                    if epoch % training.show_epoch == 0 or is_first_iteration:
                        summary.show_last()
                        last_epoch_shown = epoch

                    if epoch_end_signal is not None:
                        epoch_end_signal(self)

                    is_first_iteration = False

                except StopNetworkTraining as err:
                    # TODO: This notification breaks table view in terminal.
                    # I need to show it in a different way.
                    logs.message("TRAIN", "Epoch #{} stopped. {}"
                                          "".format(epoch, str(err)))
                    break

            if epoch != last_epoch_shown:
                summary.show_last()

            if train_end_signal is not None:
                train_end_signal(self)

            summary.finish()
            logs.newline()

        logs.message("TRAIN", "Trainig finished")



import numpy as np

from neupy.core.properties import NumberProperty
from neupy.core.base import BaseSkeleton
from neupy.utils import as_array2d, NotTrainedException


__all__ = ('ZCA',)


class ZCA(BaseSkeleton):
    """ ZCA (zero-phase component analysis) whitening.

    Parameters
    ----------
    regularization : float
        Regularization parameter. Defaults to ``1e-5``.

    Attributes
    ----------
    mean : 1D array
        Mean for each feature.
    components : array-like
        ZCA components.

    Methods
    -------
    train(data)
        Train ZCA.
    transform(data)
        Transform input data.
    """
    regularization = NumberProperty(default=1e-5, minval=0)

    def __init__(self, regularization=1e-5, **options):
        self.regularization = regularization
        self.mean = None
        self.components = None
        super(ZCA, self).__init__(**options)

    def fit(self, X, *args, **kwargs):
        """ This method is an alias to `train` method.
        This method is important for the scikit-learn
        compatibility.

        Parameters
        ----------
        X : array-like

        Returns
        -------
        ZCA class instance
        """
        self.train(X, *args, **kwargs)
        return self

    def train(self, data):
        """ Train ZCA.

        Parameters
        ----------
        data : array-like
        """
        data = as_array2d(data)
        self.mean = data.mean(axis=0)
        data = data - self.mean

        n_features = data.shape[1]
        sigma = np.dot(data.T, data) / n_features
        U, S, V = np.linalg.svd(sigma)

        self.components = (U / np.sqrt(S + self.regularization)).dot(U.T)

    def transform(self, data):
        """ Apply ZCA transformation on data.

        Parameters
        ----------
        data : array-like

        Returns
        -------
        array-like
        """
        if self.mean is None or self.components is None:
            raise NotTrainedException("Train ZCA before use it.")

        data_shape = data.shape
        data = as_array2d(data)
        data_transformed = data - self.mean
        data_transformed = np.dot(data_transformed, self.components.T)
        return data_transformed.reshape(data_shape)



import theano
import theano.tensor as T
import numpy as np

from neupy.utils import asfloat
from neupy.core.properties import ProperFractionProperty, NumberProperty
from .base import MinibatchGradientDescent


__all__ = ('RMSProp',)


class RMSProp(MinibatchGradientDescent):
    """ RMSProp algorithm.

    Parameters
    ----------
    decay : float
        Decay rate. Value need to be between ``0`` and ``1``.
        Defaults to ``0.95``.
    epsilon : float
        Value need to be greater than ``0``. Defaults to ``1e-5``.
    {MinibatchGradientDescent.batch_size}
    {GradientDescent.addons}
    {ConstructableNetwork.connection}
    {ConstructableNetwork.error}
    {BaseNetwork.step}
    {BaseNetwork.show_epoch}
    {BaseNetwork.shuffle_data}
    {BaseNetwork.epoch_end_signal}
    {BaseNetwork.train_end_signal}
    {Verbose.verbose}

    Methods
    -------
    {BaseSkeleton.predict}
    {SupervisedLearning.train}
    {BaseSkeleton.fit}
    """
    decay = ProperFractionProperty(default=0.95)
    epsilon = NumberProperty(default=1e-5, minval=0)

    def init_layers(self):
        super(RMSProp, self).init_layers()
        for layer in self.layers:
            for parameter in layer.parameters:
                parameter_shape = T.shape(parameter).eval()
                parameter.prev_mean_squred_grad = theano.shared(
                    name="prev_mean_squred_grad_" + parameter.name,
                    value=asfloat(np.zeros(parameter_shape)),
                )

    def init_param_updates(self, layer, parameter):
        prev_mean_squred_grad = parameter.prev_mean_squred_grad
        step = self.variables.step
        gradient = T.grad(self.variables.error_func, wrt=parameter)

        mean_squred_grad = (
            self.decay * prev_mean_squred_grad +
            (1 - self.decay) * gradient ** 2
        )
        parameter_delta = gradient / T.sqrt(mean_squred_grad + self.epsilon)

        return [
            (prev_mean_squred_grad, mean_squred_grad),
            (parameter, parameter - step * parameter_delta),
        ]



from abc import abstractmethod

from neupy.utils import preformat_value
from neupy.helpers.logs import Verbose
from .config import ConfigurableABC


__all__ = ('BaseSkeleton',)


class BaseSkeleton(ConfigurableABC, Verbose):
    """ Base class for neural network algorithms.

    Methods
    -------
    fit(\*args, \*\*kwargs)
        The same as ``train`` method.
    predict(input_data)
        Predict value.
    """

    def get_params(self, deep=False):
        options = {}
        for property_name, option in self.options.items():
            value = getattr(self, property_name)
            property_ = option.value

            # Options that have choices contains values that would
            # be invalid after parameter initialization
            is_choices_option = hasattr(option.value, 'choices')
            if is_choices_option and value in property_.choices.values():
                choices = {v: k for k, v in property_.choices.items()}
                value = choices[value]

            # Default value is not always valid type. For this reason we
            # need to ignore all the values that have the same value as
            # in default attibute.
            if value != property_.default:
                options[property_name] = value

        return options

    def set_params(self, **params):
        self.__dict__.update(params)
        return self

    @abstractmethod
    def train(self, input_data, target_data):
        pass

    def predict(self, input_data):
        pass

    def fit(self, X, y, *args, **kwargs):
        self.train(X, y, *args, **kwargs)
        return self

    def class_name(self):
        return self.__class__.__name__

    def _repr_options(self):
        options = []
        for option_name in self.options:
            option_value = getattr(self, option_name)
            option_value = preformat_value(option_value)

            option_repr = "{}={}".format(option_name, option_value)
            options.append(option_repr)

        return ', '.join(options)

    def __repr__(self):
        class_name = self.class_name()
        available_options = self._repr_options()
        return "{}({})".format(class_name, available_options)



import theano
import theano.typed_list
import theano.tensor as T

from neupy.core.properties import BoundedProperty
from neupy.utils import asfloat
from neupy.algorithms.gd import NoStepSelection
from neupy.algorithms.utils import (parameters2vector, count_parameters,
                                    iter_parameters, setup_parameter_updates)
from .base import GradientDescent


__all__ = ('Hessian',)


def find_hessian_and_gradient(error_function, parameters):
    """ Find Hessian and gradient for the Neural Network cost function.

    Parameters
    ----------
    function : Theano function
    parameters : list
        List of all Neural Network parameters.

    Returns
    -------
    Theano function
    """
    n_parameters = T.sum([parameter.size for parameter in parameters])
    gradients = T.grad(error_function, wrt=parameters)
    full_gradient = T.concatenate([grad.flatten() for grad in gradients])

    def find_hessian(i, full_gradient, *parameters):
        second_derivatives = []
        g = full_gradient[i]
        for parameter in parameters:
            second_derivative = T.grad(g, wrt=parameter)
            second_derivatives.append(second_derivative.flatten())

        return T.concatenate(second_derivatives)

    hessian, _ = theano.scan(
        find_hessian,
        sequences=T.arange(n_parameters),
        non_sequences=[full_gradient] + parameters,
    )
    hessian_matrix = hessian.reshape((n_parameters, n_parameters))

    return hessian_matrix, full_gradient


class Hessian(NoStepSelection, GradientDescent):
    """ Hessian gradient decent optimization. This GD algorithm
    variation using second derivative information helps choose better
    gradient direction and as a consequence better weight update
    parameter after eqch epoch.

    Parameters
    ----------
    penalty_const : float
        Inverse hessian could be singular matrix. For this reason
        algorithm include penalty that add to hessian matrix identity
        multiplied by defined constant. Defaults to ``1``.
    {GradientDescent.addons}
    {ConstructableNetwork.connection}
    {ConstructableNetwork.error}
    {BaseNetwork.show_epoch}
    {BaseNetwork.shuffle_data}
    {BaseNetwork.epoch_end_signal}
    {BaseNetwork.train_end_signal}
    {Verbose.verbose}

    Methods
    -------
    {BaseSkeleton.predict}
    {SupervisedLearning.train}
    {BaseSkeleton.fit}

    See Also
    --------
    :network:`HessianDiagonal` : Hessian diagonal approximation.
    """
    penalty_const = BoundedProperty(default=1, minval=0)

    def init_train_updates(self):
        n_parameters = count_parameters(self)
        parameters = list(iter_parameters(self))
        param_vector = parameters2vector(self)
        penalty_const = asfloat(self.penalty_const)

        hessian_matrix, full_gradient = find_hessian_and_gradient(
            self.variables.error_func, parameters
        )
        hessian_inverse = T.nlinalg.matrix_inverse(
            hessian_matrix + penalty_const * T.eye(n_parameters)
        )

        updated_parameters = param_vector - hessian_inverse.dot(full_gradient)
        updates = setup_parameter_updates(parameters, updated_parameters)

        return updates



from operator import mul

import numpy as np
from numpy.linalg import norm

from neupy.core.properties import (IntProperty, TypedListProperty,
                                   ChoiceProperty)
from neupy.utils import format_data
from neupy.algorithms import Kohonen


__all__ = ('SOFM',)


def neuron_neighbours(neurons, center, radius):
    """ Function find all neighbours neurons by radius and coords.

    Parameters
    ----------
    neurons : arary-like
        Array element with neurons.
    center : tuple
        Index of the main neuron for which function must find neighbours.
    radius : int
        Radius indetify which neurons hear the main one are neighbours.

    Returns
    -------
    array-like
        Return matrix with the same dimention as ``neurons`` where center
        element and it neighbours positions filled with value ``1``
        and other as ``0``.

    Examples
    --------
    >>> import numpy as np
    >>> from neupy.algorithms.competitive.sofm import neuron_neighbours
    >>>
    >>> neuron_neighbours(np.zeros((3, 3)), (0, 0), 1)
    array([[ 1.,  1.,  0.],
           [ 1.,  0.,  0.],
           [ 0.,  0.,  0.]])
    >>> neuron_neighbours(np.zeros((5, 5)), (2, 2), 2)
    array([[ 0.,  0.,  1.,  0.,  0.],
           [ 0.,  1.,  1.,  1.,  0.],
           [ 1.,  1.,  1.,  1.,  1.],
           [ 0.,  1.,  1.,  1.,  0.],
           [ 0.,  0.,  1.,  0.,  0.]])
    """
    center_x, center_y = center
    nx, ny = neurons.shape

    y, x = np.ogrid[-center_x:nx - center_x, -center_y:ny - center_y]
    mask = (x * x + y * y) <= radius ** 2
    neurons[mask] = 1

    return neurons


def dot_product(input_data, weight):
    """ Simple dot product.

    Parameters
    ----------
    input_data : array-like
    weight : array-like

    Returns
    -------
    array-like
    """
    return input_data.dot(weight)


def neg_euclid_distance(input_data, weight):
    """ Negative Euclidian distance between input
    data and weight.

    Parameters
    ----------
    input_data : array-like
    weight : array-like

    Returns
    -------
    array-like
    """
    euclid_dist = norm(input_data.T - weight, axis=0)
    return -np.reshape(euclid_dist, (1, weight.shape[1]))


def cosine_similarity(input_data, weight):
    """ Cosine similarity between input data and weight.

    Parameters
    ----------
    input_data : array-like
    weight : array-like

    Returns
    -------
    array-like
    """
    norm_prod = norm(input_data) * norm(weight, axis=0)
    summated_data = np.dot(input_data, weight)
    cosine_dist = summated_data / norm_prod
    return np.reshape(cosine_dist, (1, weight.shape[1]))


class SOFM(Kohonen):
    """ Self-Organizing Feature Map.

    Parameters
    ----------
    learning_radius : int
        Learning radius.
    features_grid : int
        Learning radius.
    transform : {{'linear', 'euclid', 'cos'}}
        Indicate transformation operation related to the input layer.
        The ``linear`` value mean that input data would be multiplied by
        weights in typical way. The ``euclid`` method will identify the
        closest weight vector to the input one. The ``cos`` made the same
        as ``euclid``, but instead of euclid distance it uses cosine
        similarity. Defaults to ``linear``.
    {BaseAssociative.n_inputs}
    {BaseAssociative.n_outputs}
    {BaseAssociative.weight}
    {BaseNetwork.step}
    {BaseNetwork.show_epoch}
    {BaseNetwork.shuffle_data}
    {BaseNetwork.epoch_end_signal}
    {BaseNetwork.train_end_signal}
    {Verbose.verbose}

    Methods
    -------
    {BaseSkeleton.predict}
    {BaseAssociative.train}
    {BaseSkeleton.fit}
    """

    learning_radius = IntProperty(default=0, minval=0)
    features_grid = TypedListProperty()
    transform = ChoiceProperty(default='linear', choices={
        'linear': dot_product,
        'euclid': neg_euclid_distance,
        'cos': cosine_similarity,
    })

    def __init__(self, **options):
        super(SOFM, self).__init__(**options)

        invalid_feature_grid = (
            self.features_grid is not None and
            mul(*self.features_grid) != self.n_outputs
        )
        if invalid_feature_grid:
            raise ValueError(
                "Feature grid should contain the same number of elements as "
                "in the output layer: {0}, but found: {1} ({2}x{3})"
                "".format(
                    self.n_outputs,
                    mul(*self.features_grid),
                    self.features_grid[0],
                    self.features_grid[1]
                )
            )

    def init_properties(self):
        super(SOFM, self).init_properties()

        if self.features_grid is None:
            self.features_grid = (self.n_outputs, 1)

    def predict_raw(self, input_data):
        input_data = format_data(input_data)
        output = np.zeros((input_data.shape[0], self.n_outputs))
        for i, input_row in enumerate(input_data):
            output[i, :] = self.transform(input_row.reshape(1, -1),
                                          self.weight)
        return output

    def update_indexes(self, layer_output):
        neuron_winner = layer_output.argmax(axis=1)
        feature_bound = self.features_grid[1]

        output_with_neightbours = neuron_neighbours(
            np.reshape(layer_output, self.features_grid),
            (neuron_winner // feature_bound, neuron_winner % feature_bound),
            self.learning_radius
        )
        index_y, _ = np.nonzero(
            np.reshape(
                output_with_neightbours,
                (self.n_outputs, 1)
            )
        )
        return index_y



from neupy.utils import is_list_of_integers
from neupy.layers.connections import NetworkConnectionError, LayerConnection
from neupy.network import ConstructableNetwork
from neupy.layers import Step, Output


__all__ = ('BaseLinearNetwork',)


class BaseLinearNetwork(ConstructableNetwork):
    """ Base class for feedforward neural network without hidden layers.

    Notes
    -----
    * Input layer should be :layer:`Step` class instance.

    Parameters
    ----------
    connection : list, tuple or object
        Should be a list or tuple that contains two integers. First integer
        describe number of input units and the seconds one number of output
        units.
    {ConstructableNetwork.error}
    {BaseNetwork.step}
    {BaseNetwork.show_epoch}
    {BaseNetwork.shuffle_data}
    {BaseNetwork.epoch_end_signal}
    {BaseNetwork.train_end_signal}
    {Verbose.verbose}

    Methods
    -------
    {BaseSkeleton.predict}
    {SupervisedLearning.train}
    {BaseSkeleton.fit}
    """

    def __init__(self, connection, **options):
        if len(connection) != 2:
            raise ValueError("This network should contains two layers.")

        if is_list_of_integers(connection):
            input_layer_size, output_layer_size = connection
            connection = Step(input_layer_size) > Output(output_layer_size)

        if not isinstance(connection, LayerConnection):
            raise ValueError("Invalid network connection structure.")

        if not isinstance(connection.input_layer, Step):
            raise NetworkConnectionError(
                "Input layer should contains step activation function "
                "(``Step`` class instance)."
            )

        super(BaseLinearNetwork, self).__init__(connection, **options)



import re
from abc import ABCMeta

from six import with_metaclass

from neupy.utils import AttributeKeyDict


__all__ = ("SharedDocsMeta", "SharedDocs", "SharedDocsException",
           "SharedDocsABCMeta")


def merge_dicts(left_dict, right_dict):
    """ Merge two dictionaries in one.

    Parameters
    ----------
    left_dict : dict
    right_dict : dict

    Returns
    -------
    dict
    """
    return dict(left_dict, **right_dict)


def find_numpy_doc_indent(docs):
    """ Find indent for Numpy styled documentation and return number of
    shifts inside of it.

    Parameters
    ----------
    docs : str

    Returns
    -------
    int or None
        Returns number of indentations in documentation. If it doesn't
        identify indentation function output will be ``None`` value.
    """

    indent_detector = re.compile(r"(?P<indent>\ *)(?P<dashes>-{3,})")
    indent_info = indent_detector.findall(docs)

    if indent_info:
        indent, _ = indent_info[0]
        return len(indent)


def iter_parameters(docs):
    """ Find parameters described in the Numpy style documentation.

    Parameters
    ----------
    docs : str

    Yields
    ------
    tuple
        Yields tuple that contain 3 values. There are: parameter name,
        parameter type and parameter description.
    """

    n_indents = find_numpy_doc_indent(docs)
    doc_indent = ' ' * n_indents if n_indents else ''
    parser = re.compile(
        r"(?P<name>\w+?)\s*\:\s*(?P<type>[^\n]+)"
        r"((?P<description>(\n{indent}\ +[^\n]+)|(\n))*)"
        "".format(indent=doc_indent)
    )

    for name, type_, desc, _, _, _ in parser.findall(docs):
        yield (name, type_, desc)


def iter_methods(docs):
    """ Find methods described in the Numpy style documentation.

    Parameters
    ----------
    docs : str

    Yields
    ------
    tuple
        Yields tuple that contain 3 values. There are: method name,
        method parameters and method description.
    """

    n_indents = find_numpy_doc_indent(docs)
    doc_indent = ' ' * n_indents if n_indents else ''
    parser = re.compile(
        r"(?P<name>\w+?)(\((.+?)?\))"
        r"((?P<description>\n{indent}\ +[^\n]+)*)"
        "".format(indent=doc_indent)
    )

    for name, func_params, _, desc, _ in parser.findall(docs):
        yield (name, func_params, desc)


def parse_warns(docs):
    """ Find warning described in the Numpy style documentation.

    Parameters
    ----------
    docs : str

    Returns
    -------
    str or None
        Returns warnings from documentation or ``None`` if function
        didn't find it.
    """

    parser = re.compile(r"Warns\s+-+\s+(?P<warntext>(.+\n)+)")
    doc_warns = parser.findall(docs)

    if not doc_warns:
        return None

    doc_warns, _ = doc_warns[0]
    return doc_warns


class SharedDocsException(Exception):
    """ Exception that help identify problems related to shared
    documentation.
    """


class SharedDocsMeta(type):
    """ Meta-class for shared documentation. This class conatains main
    functionality that help inherit parameters and methods descriptions
    from parent classes. This class automaticaly format class documentation
    using basic python format syntax for objects.
    """

    def __new__(cls, clsname, bases, attrs):
        new_class = super(SharedDocsMeta, cls).__new__(cls, clsname,
                                                       bases, attrs)
        if new_class.__doc__ is None:
            return new_class

        class_docs = new_class.__doc__
        n_indents = find_numpy_doc_indent(class_docs)

        if n_indents is None:
            return new_class

        parameters = {}
        parent_classes = new_class.__mro__

        for parent_class in parent_classes:
            parent_docs = parent_class.__doc__

            if parent_docs is None:
                continue

            parent_name = parent_class.__name__
            parent_params = parameters[parent_name] = AttributeKeyDict()

            for name, type_, desc in iter_parameters(parent_docs):
                parent_params[name] = "{} : {}{}".format(name, type_, desc)

            for name, func_params, desc in iter_methods(parent_docs):
                parent_params[name] = ''.join([name, func_params, desc])

            doc_warns = parse_warns(parent_docs)
            if doc_warns is not None:
                parent_params['Warns'] = doc_warns

        try:
            new_class.__doc__ = new_class.__doc__.format(**parameters)
        except Exception as exception:
            exception_classname = exception.__class__.__name__
            raise SharedDocsException(
                "Can't format documentation for class `{}`. "
                "Catched `{}` exception with message: {}".format(
                    new_class.__name__,
                    exception_classname,
                    exception
                )
            )

        return new_class


class SharedDocsABCMeta(SharedDocsMeta, ABCMeta):
    """ Meta-class that combine ``SharedDocsMeta`` and ``ABCMeta``
    meta-classes.
    """


class SharedDocs(with_metaclass(SharedDocsMeta)):
    """ Main class that provide with shared documentation functionality.
    """



from __future__ import division

import theano.tensor as T
from neupy.utils import smallest_positive_number


__all__ = ('mse', 'rmse', 'mae', 'msle', 'rmsle', 'binary_crossentropy',
           'categorical_crossentropy')


def mse(expected, predicted):
    """ Mean squared error.
    """
    return T.square(predicted - expected).mean()


def rmse(expected, predicted):
    """ Root mean squared error.
    """
    return T.sqrt(mse(expected, predicted))


def mae(expected, predicted):
    """ Mean absolute error.
    """
    return T.abs_(expected - predicted).mean()


def msle(expected, predicted):
    """ Mean squared logarithmic error.
    """
    squared_log = (T.log(predicted + 1) - T.log(expected + 1)) ** 2
    return squared_log.mean()


def rmsle(expected, predicted):
    """ Root mean squared logarithmic error.
    """
    return T.sqrt(msle(expected, predicted))


def binary_crossentropy(expected, predicted):
    """ Binary cross-entropy error.
    """
    epsilon = smallest_positive_number()
    predicted = T.clip(predicted, epsilon, 1.0 - epsilon)
    return T.nnet.binary_crossentropy(predicted, expected).mean()


def categorical_crossentropy(expected, predicted):
    """ Categorical cross-entropy error.
    """
    epsilon = smallest_positive_number()
    predicted = T.clip(predicted, epsilon, 1.0 - epsilon)
    return T.nnet.categorical_crossentropy(predicted, expected).mean()



import time
import types

import theano
import theano.sparse
import theano.tensor as T

from neupy.utils import (AttributeKeyDict, asfloat, is_list_of_integers,
                         format_data, does_layer_accept_1d_feature)
from neupy.layers import BaseLayer, Output, Dropout
from neupy.layers.utils import generate_layers
from neupy.core.properties import ChoiceProperty
from neupy.layers.connections import LayerConnection, NetworkConnectionError
from neupy.network import errors
from .learning import SupervisedLearning
from .base import BaseNetwork


__all__ = ('ConstructableNetwork',)


def clean_layers(connection):
    """ Clean layers connections and format transform them into one format.
    Also this function validate layers connections.

    Parameters
    ----------
    connection : list, tuple or object
        Layers connetion in different formats.

    Returns
    -------
    object
        Cleaned layers connection.
    """

    if is_list_of_integers(connection):
        connection = generate_layers(list(connection))

    if isinstance(connection, tuple):
        connection = list(connection)

    islist = isinstance(connection, list)

    if islist and isinstance(connection[0], BaseLayer):
        chain_connection = connection.pop()
        for layer in reversed(connection):
            chain_connection = LayerConnection(layer, chain_connection)
        connection = chain_connection

    elif islist and isinstance(connection[0], LayerConnection):
        pass

    if not isinstance(connection.output_layer, Output):
        raise NetworkConnectionError("Final layer must be Output class "
                                     "instance.")

    return connection


def create_input_variable(input_layer, variable_name):
    """ Create input variable based on input layer information.

    Parameters
    ----------
    input_layer : object
    variable_name : str

    Returns
    -------
    Theano variable
    """
    dim_to_variable_type = {
        2: T.matrix,
        3: T.tensor3,
        4: T.tensor4,
    }
    ndim = input_layer.weight.ndim

    if ndim not in dim_to_variable_type:
        raise ValueError("Layer's input needs to be 2, 3 or 4 dimensional. "
                         "Found {}".format(ndim))

    variable_type = dim_to_variable_type[ndim]
    return variable_type(variable_name)


def create_output_variable(error_function, variable_name):
    """ Create output variable based on error function.

    Parameters
    ----------
    error_function : function
    variable_name : str

    Returns
    -------
    Theano variable
    """
    # TODO: Solution is not user friendly. I need to find
    # better solution later.
    if hasattr(error_function, 'expected_dtype'):
        network_output_dtype = error_function.expected_dtype
    else:
        network_output_dtype = T.matrix

    return network_output_dtype(variable_name)


def find_input_layer(layers):
    """ Function checks list of layer and finds an input layer.

    Parameters
    ----------
    layers : iterative object
        Ordered list of layers.

    Returns
    -------
    BaseLayer instance or None
        Function will return input layer if it exists in the specified
        connection structure. Otherwise, output will be equal to ``None``.
    """
    for layer in layers:
        if not isinstance(layer, Dropout):
            return layer


class ErrorFunctionProperty(ChoiceProperty):
    """ Property that helps select error function from
    available or define a new one.

    Parameters
    ----------
    {ChoiceProperty.choices}
    {BaseProperty.default}
    {BaseProperty.required}
    """
    def __set__(self, instance, value):
        if isinstance(value, types.FunctionType):
            return super(ChoiceProperty, self).__set__(instance, value)
        return super(ErrorFunctionProperty, self).__set__(instance, value)

    def __get__(self, instance, value):
        founded_value = super(ChoiceProperty, self).__get__(instance, value)
        if isinstance(founded_value, types.FunctionType):
            return founded_value
        return super(ErrorFunctionProperty, self).__get__(instance,
                                                          founded_value)


class ConstructableNetwork(SupervisedLearning, BaseNetwork):
    """ Class contains functionality that helps work with network that have
    constructable layers architecture.

    Parameters
    ----------
    connection : list, tuple or object
        Network architecture. That variables could be described in
        different ways. The simples one is a list or tuple that contains
        integers. Each integer describe layer input size. For example,
        ``(2, 4, 1)`` means that network will have 3 layers with 2 input
        units, 4 hidden units and 1 output unit. The one limitation of that
        method is that all layers automaticaly would with sigmoid actiavtion
        function. Other way is just a list of ``BaseLayer``` class
        instances. For example: ``[Tanh(2), Relu(4), Output(1)].
        And the most readable one is just layer pipeline
        ``Tanh(2) > Relu(4) > Output(1)``.
    error : {{'mse', 'rmse', 'mae', 'categorical_crossentropy', \
    'binary_crossentropy'}} or function
        Function that calculate prediction error.
        Defaults to ``mse``.

        * ``mae`` - Mean Absolute Error.

        * ``mse`` - Mean Squared Error.

        * ``rmse`` - Root Mean Squared Error.

        * ``msle`` - Mean Squared Logarithmic Error.

        * ``rmsle`` - Root Mean Squared Logarithmic Error.

        * ``categorical_crossentropy`` - Categorical cross entropy.

        * ``binary_crossentropy`` - Binary cross entropy.

        * Custom function that accept two mandatory arguments.
        The first one is expected value and the second one is
        predicted value. Example: ``custom_func(expected, predicted)``
    {BaseNetwork.step}
    {BaseNetwork.show_epoch}
    {BaseNetwork.shuffle_data}
    {BaseNetwork.epoch_end_signal}
    {BaseNetwork.train_end_signal}
    {Verbose.verbose}

    Attributes
    ----------
    {BaseNetwork.errors}
    {BaseNetwork.train_errors}
    {BaseNetwork.validation_errors}
    {BaseNetwork.last_epoch}
    """
    error = ErrorFunctionProperty(default='mse', choices={
        'mae': errors.mae,
        'mse': errors.mse,
        'rmse': errors.rmse,
        'msle': errors.msle,
        'rmsle': errors.rmsle,
        'binary_crossentropy': errors.binary_crossentropy,
        'categorical_crossentropy': errors.categorical_crossentropy,
    })

    def __init__(self, connection, *args, **kwargs):
        self.connection = clean_layers(connection)

        self.all_layers = list(self.connection)
        self.layers = self.all_layers[:-1]

        self.input_layer = find_input_layer(self.layers)
        self.hidden_layers = self.layers[1:]
        self.output_layer = self.all_layers[-1]

        self.init_layers()
        super(ConstructableNetwork, self).__init__(*args, **kwargs)

        self.logs.message("THEANO", "Initializing Theano variables and "
                                    "functions.")
        start_init_time = time.time()

        self.variables = AttributeKeyDict(
            network_input=create_input_variable(
                self.input_layer, variable_name='x'
            ),
            network_output=create_output_variable(
                self.error, variable_name='y'
            ),
        )
        self.methods = AttributeKeyDict()

        self.init_variables()
        self.init_methods()

        finish_init_time = time.time()
        self.logs.message("THEANO", "Initialization finished sucessfully. "
                          "It took {:.2f} seconds"
                          "".format(finish_init_time - start_init_time))

    def init_variables(self):
        """ Initialize Theano variables.
        """
        network_input = self.variables.network_input
        network_output = self.variables.network_output

        train_prediction = prediction = network_input
        for layer in self.layers:
            if not isinstance(layer, Dropout):
                prediction = layer.output(prediction)
            train_prediction = layer.output(train_prediction)

        self.variables.update(
            step=theano.shared(name='step', value=asfloat(self.step)),
            epoch=theano.shared(name='epoch', value=asfloat(self.last_epoch)),

            prediction_func=prediction,
            train_prediction_func=train_prediction,

            error_func=self.error(network_output, train_prediction),
            validation_error_func=self.error(network_output, prediction),
        )

    def init_methods(self):
        """ Initialize all methods that needed for prediction and
        training procedures.
        """
        network_input = self.variables.network_input
        network_output = self.variables.network_output

        self.methods.predict_raw = theano.function(
            inputs=[self.variables.network_input],
            outputs=self.variables.prediction_func
        )
        self.methods.train_epoch = theano.function(
            inputs=[network_input, network_output],
            outputs=self.variables.error_func,
            updates=self.init_train_updates(),
        )
        self.methods.prediction_error = theano.function(
            inputs=[network_input, network_output],
            outputs=self.variables.validation_error_func
        )

    def init_layers(self):
        """ Initialize layers in the same order as they were list in
        network initialization step.
        """
        for layer in self.layers:
            layer.initialize()

    def init_train_updates(self):
        """ Initialize train function update in Theano format that
        would be trigger after each training epoch.
        """
        updates = []
        for layer in self.layers:
            updates.extend(self.init_layer_updates(layer))
        return updates

    def init_layer_updates(self, layer):
        """ Initialize train function update in Theano format that
        would be trigger after each training epoch for each layer.

        Parameters
        ----------
        layer : object
            Any layer that inherit from BaseLayer class.

        Returns
        -------
        list
            Update that excaptable by ``theano.function``. There should be
            a lits that contains tuples with 2 elements. First one should
            be parameter that would be updated after epoch and the second one
            should update rules for this parameter. For example parameter
            could be a layer's weight and bias.
        """
        updates = []
        for parameter in layer.parameters:
            updates.extend(self.init_param_updates(layer, parameter))
        return updates

    def init_param_updates(self, parameter):
        """ Initialize parameter updates.

        Parameters
        ----------
        parameter : object
            Usualy it is a weight or bias.

        Returns
        -------
        list
            List of updates related to the specified parameter.
        """
        return []

    def format_input_data(self, input_data):
        """ Input data format is depence on the input layer
        structure.

        Parameters
        ----------
        input_data : array-like or None

        Returns
        -------
        array-like or None
            Function returns formatted array.
        """
        if input_data is not None:
            is_feature1d = does_layer_accept_1d_feature(self.input_layer)
            return format_data(input_data, is_feature1d)

    def format_target_data(self, target_data):
        """ Target data format is depence on the output layer
        structure.

        Parameters
        ----------
        target_data : array-like or None

        Returns
        -------
        array-like or None
            Function returns formatted array.
        """
        if target_data is not None:
            is_feature1d = does_layer_accept_1d_feature(self.output_layer)
            return format_data(target_data, is_feature1d)

    def prediction_error(self, input_data, target_data):
        """ Calculate prediction accuracy for input data.

        Parameters
        ----------
        input_data : array-like
        target_data : array-like

        Returns
        -------
        float
            Prediction error.
        """
        return self.methods.prediction_error(
            self.format_input_data(input_data),
            self.format_target_data(target_data)
        )

    def predict_raw(self, input_data):
        """ Make raw prediction without final layer postprocessing step.

        Parameters
        ----------
        input_data : array-like

        Returns
        -------
        array-like
        """
        input_data = self.format_input_data(input_data)
        return self.methods.predict_raw(input_data)

    def predict(self, input_data):
        """ Return prediction results for the input data. Output result also
        include postprocessing step related to the final layer that
        transform output to convenient format for end-use.

        Parameters
        ----------
        input_data : array-like

        Returns
        -------
        array-like
        """
        raw_prediction = self.predict_raw(input_data)
        return self.output_layer.output(raw_prediction)

    def on_epoch_start_update(self, epoch):
        """ Function would be trigger before run all training procedure
        related to the current epoch.

        Parameters
        ----------
        epoch : int
            Current epoch number.
        """
        super(ConstructableNetwork, self).on_epoch_start_update(epoch)
        self.variables.epoch.set_value(epoch)

    def train(self, input_train, target_train, input_test=None,
              target_test=None, *args, **kwargs):
        """ Trains neural network.
        """
        return super(ConstructableNetwork, self).train(
            self.format_input_data(input_train),
            self.format_target_data(target_train),
            self.format_input_data(input_test),
            self.format_target_data(target_test),
            *args, **kwargs
        )

    def train_epoch(self, input_train, target_train):
        """ Trains neural network over one epoch.

        Parameters
        ----------
        input_data : array-like
        target_data : array-like

        Returns
        -------
        float
            Prediction error.
        """
        return self.methods.train_epoch(input_train, target_train)

    def architecture(self):
        """ Shows network's architecture in the terminal if
        ``verbose`` parameter is equal to ``True``.
        """
        self.logs.title("Network's architecture")

        for i, layer in enumerate(self.all_layers, start=1):
            self.logs.write("{:>3}. {}".format(i, str(layer)))

        self.logs.newline()

    def __repr__(self):
        return "{}({}, {})".format(self.class_name(), self.connection,
                                   self._repr_options())



import inspect

import theano
import theano.tensor as T
from theano.tensor.var import TensorVariable
from theano.tensor.sharedvar import TensorSharedVariable
import numpy as np
from scipy.sparse import issparse


__all__ = ('format_data', 'does_layer_accept_1d_feature', 'asfloat',
           'AttributeKeyDict', 'is_list_of_integers', 'preformat_value',
           'as_array2d', 'NotTrainedException', 'smallest_positive_number')


class NotTrainedException(Exception):
    """ Exception needs for cases when algorithm is not trained
    and can not be applied.
    """


def format_data(data, is_feature1d=True, copy=False):
    """ Transform data in a standardized format.

    Notes
    -----
    It should be applied to the input data prior to use in
    learning algorithms.

    Parameters
    ----------
    data : array-like
        Data that should be formated. That could be, matrix, vector or
        Pandas DataFrame instance.
    is_feature1d : bool
        Should be equal to ``True`` if input data if a vector that
        contains n samples with 1 feature each. Defaults to ``True``.
    copy : bool
        Defaults to ``False``.

    Returns
    -------
    ndarray
        The same input data but transformed to a standardized format
        for further use.
    """
    if data is None or issparse(data):
        return data

    data = asfloat(data)

    if not isinstance(data, np.ndarray) or copy:
        data = np.array(data, copy=copy)

    # Valid number of features for one or two dimentions
    n_features = data.shape[-1]

    if data.ndim == 1:
        data_shape = (n_features, 1) if is_feature1d else (1, n_features)
        data = data.reshape(data_shape)

    return data


def does_layer_accept_1d_feature(layer):
    """ Check if 1D feature values are valid for the layer.

    Parameters
    ----------
    layer : object

    Returns
    -------
    bool
    """
    return (layer.size == 1)


def asfloat(value):
    """ Convert variable to float type configured by theano
    floatX variable.

    Parameters
    ----------
    value : matrix, ndarray, Theano variable or scalar
        Value that could be converted to float type.

    Returns
    -------
    matrix, ndarray, Theano variable or scalar
        Output would be input value converted to float type
        configured by theano floatX variable.
    """
    float_type = theano.config.floatX

    if isinstance(value, (np.matrix, np.ndarray)):
        if value.dtype != np.dtype(float_type):
            return value.astype(float_type)
        else:
            return value

    elif isinstance(value, (TensorVariable, TensorSharedVariable)):
        return T.cast(value, float_type)

    elif issparse(value):
        return value

    float_x_type = np.cast[float_type]
    return float_x_type(value)


def asint(value):
    """ Convert variable to an integer type. Number of bits per
    integer depence on floatX Theano variable.

    Parameters
    ----------
    value : matrix, ndarray, Theano variable or scalar
        Value that could be converted to the integer type.

    Returns
    -------
    matrix, ndarray, Theano variable or scalar
        Output would be input value converted to the integer type.
    """
    int2float_types = {
        'float32': 'int32',
        'float64': 'int64',
    }

    float_type = theano.config.floatX
    int_type = int2float_types[float_type]

    if isinstance(value, (np.matrix, np.ndarray)):
        if value.dtype != np.dtype(int_type):
            return value.astype(int_type)
        else:
            return value

    elif isinstance(value, (TensorVariable, TensorSharedVariable)):
        return T.cast(value, int_type)

    elif issparse(value):
        return value

    int_x_type = np.cast[int_type]
    return int_x_type(value)


class AttributeKeyDict(dict):
    """ Modified built-in Python ``dict`` class. That modification
    helps get and set values like attributes.

    Examples
    --------
    >>> attrdict = AttributeKeyDict()
    >>> attrdict
    {}
    >>> attrdict.test_key = 'test_value'
    >>> attrdict
    {'test_key': 'test_value'}
    >>> attrdict.test_key
    'test_value'
    """

    def __getattr__(self, attrname):
        return self[attrname]

    def __setattr__(self, attrname, value):
        self[attrname] = value

    def __delattr__(self, attrname):
        del self[attrname]


def is_list_of_integers(sequence):
    """ Check that sequence contains only integer numbers.

    Parameters
    ----------
    sequence : list, tuple
        Array that should be validated.

    Returns
    -------
    bool
        Result would be ``True`` only if each element in a sequence contains
        is an integer. ``False`` otherwise.
    """
    return all(isinstance(element, int) for element in sequence)


def preformat_value(value):
    """ Function pre-format input value depence on it's type.

    Parameters
    ----------
    value : object

    Returns
    -------
    object
    """
    if inspect.isfunction(value) or inspect.isclass(value):
        return value.__name__

    elif isinstance(value, (list, tuple, set)):
        return [preformat_value(v) for v in value]

    elif isinstance(value, (np.ndarray, np.matrix)):
        return value.shape

    return value


def as_array2d(array):
    """ Transform any array to 2D.

    Parameters
    ----------
    array : array-like

    Returns
    -------
    array-like
        The same array transformed to 2D.
    """
    if array.ndim == 1:
        return array.reshape((1, -1))

    n_samples, feature_shape = array.shape[0], array.shape[1:]
    return array.reshape((n_samples, np.prod(feature_shape)))


def smallest_positive_number():
    """ Based on the `floatX` variables function returns different
    smallest positive numbers.

    Returns
    -------
    float
        Smallest positive float number.
    """
    float_type = theano.config.floatX
    epsilon_values = {
        'float32': 1e-7,
        'float64': 1e-16,
    }
    return epsilon_values[float_type]



import numpy as np
import theano
import theano.tensor as T

from neupy.utils import asfloat
from neupy.core.config import Configurable
from neupy.core.properties import (TypedListProperty, ArrayProperty,
                                   ChoiceProperty, IntProperty)
from neupy.layers.connections import ChainConnection
from .utils import XAVIER_NORMAL, VALID_INIT_METHODS, generate_weight


__all__ = ('BaseLayer', 'ParameterBasedLayer')


class BaseLayer(ChainConnection, Configurable):
    """ Base class for all layers.
    """
    def __init__(self, *args, **options):
        super(BaseLayer, self).__init__()

        self.parameters = []

        # Default variables which will change after initialization
        self.relate_to_layer = None
        self.relate_from_layer = None
        self.layer_id = 1

        Configurable.__init__(self, **options)

    def initialize(self):
        if self.relate_from_layer is not None:
            self.layer_id = self.relate_from_layer.layer_id + 1

    def relate_to(self, right_layer):
        self.relate_to_layer = right_layer
        right_layer.relate_from_layer = self

    def __repr__(self):
        classname = self.__class__.__name__
        return '{name}()'.format(name=classname)


def create_shared_parameter(value, name, shape, init_method, bounds):
    """ Creates NN parameter as Theano shared variable.

    Parameters
    ----------
    value : array-like, theano shared variable or None
        Default value for the parameter. If value eqaul to ``None``
        parameter will be created bsaed on the ``init_method`` value.
    name : str
        Sahred variable name.
    shape : tuple
        Parameter shape.
    init_method : str
        Weight initialization procedure name.
    bounds : tuple
        Specific parameter for the one of the ``init_method``
        argument.

    Returns
    -------
    Theano shared variable.
    """
    if isinstance(value, (T.sharedvar.SharedVariable, T.Variable)):
        return value

    if value is None:
        value = generate_weight(shape, bounds, init_method)

    return theano.shared(value=asfloat(value), name=name, borrow=True)


class SharedArrayProperty(ArrayProperty):
    """ In addition to Numpy arrays and matrix property support also
    Theano shared variables.

    Parameters
    ----------
    {BaseProperty.default}
    {BaseProperty.required}
    """
    expected_type = (np.matrix, np.ndarray,
                     T.sharedvar.SharedVariable,
                     T.Variable)


class ParameterBasedLayer(BaseLayer):
    """ Layer that creates weight and bias parameters.

    Parameters
    ----------
    size : int
        Layer input size.
    weight : 2D array-like or None
        Define your layer weights. ``None`` means that your weights will be
        generate randomly dependence on property ``init_method``.
        ``None`` by default.
    bias : 1D array-like or None
        Define your layer bias. ``None`` means that your weights will be
        generate randomly dependence on property ``init_method``.
    init_method : {{'bounded', 'normal', 'ortho', 'xavier_normal',\
    'xavier_uniform', 'he_normal', 'he_uniform'}}
        Weight initialization method. Defaults to ``xavier_normal``.

        * ``normal`` will generate random weights from normal distribution \
        with standard deviation equal to ``0.01``.

        * ``bounded`` generate random weights from Uniform distribution.

        * ``ortho`` generate random orthogonal matrix.

        * ``xavier_normal`` generate random matrix from normal distrubtion \
        where variance equal to :math:`\\frac{{2}}{{fan_{{in}} + \
        fan_{{out}}}}`. Where :math:`fan_{{in}}` is a number of \
        layer input units and :math:`fan_{{out}}` - number of layer \
        output units.

        * ``xavier_uniform`` generate random matrix from uniform \
        distribution \ where :math:`w_{{ij}} \in \
        [-\\sqrt{{\\frac{{6}}{{fan_{{in}} + fan_{{out}}}}}}, \
        \\sqrt{{\\frac{{6}}{{fan_{{in}} + fan_{{out}}}}}}`].

        * ``he_normal`` generate random matrix from normal distrubtion \
        where variance equal to :math:`\\frac{{2}}{{fan_{{in}}}}`. \
        Where :math:`fan_{{in}}` is a number of layer input units.

        * ``he_uniform`` generate random matrix from uniformal \
        distribution where :math:`w_{{ij}} \in [\
        -\\sqrt{{\\frac{{6}}{{fan_{{in}}}}}}, \
        \\sqrt{{\\frac{{6}}{{fan_{{in}}}}}}]`

    bounds : tuple of two float
        Available only for ``init_method`` equal to ``bounded``.  Value
        identify minimum and maximum possible value in random weights.
        Defaults to ``(0, 1)``.
    """
    size = IntProperty(minval=1)
    weight = SharedArrayProperty(default=None)
    bias = SharedArrayProperty(default=None)
    bounds = TypedListProperty(default=(0, 1), element_type=(int, float))
    init_method = ChoiceProperty(default=XAVIER_NORMAL,
                                 choices=VALID_INIT_METHODS)

    def __init__(self, size, **options):
        if size is not None:
            options['size'] = size
        super(ParameterBasedLayer, self).__init__(**options)

    def weight_shape(self):
        output_size = self.relate_to_layer.size
        return (self.size, output_size)

    def bias_shape(self):
        output_size = self.relate_to_layer.size
        return (output_size,)

    def initialize(self):
        super(ParameterBasedLayer, self).initialize()

        self.weight = create_shared_parameter(
            value=self.weight,
            name='weight_{}'.format(self.layer_id),
            shape=self.weight_shape(),
            bounds=self.bounds,
            init_method=self.init_method,
        )
        self.bias = create_shared_parameter(
            value=self.bias,
            name='bias_{}'.format(self.layer_id),
            shape=self.bias_shape(),
            bounds=self.bounds,
            init_method=self.init_method,
        )
        self.parameters = [self.weight, self.bias]

    def __repr__(self):
        classname = self.__class__.__name__
        return '{name}({size})'.format(name=classname, size=self.size)



from .base import BaseNetwork
from .utils import StopNetworkTraining
from .constructor import ConstructableNetwork, ConstructableNetwork
from .learning import (SupervisedLearning, UnsupervisedLearning,
                       LazyLearning)



import numpy as np


__all__ = ('iter_until_converge', 'shuffle', 'normalize_error', 'step',
           'StopNetworkTraining')


class StopNetworkTraining(Exception):
    """ Exception that needs to be triggered in case of
    early training interruption.
    """


def iter_until_converge(network, epsilon, max_epochs):
    logs = network.logs
    # Trigger first iteration and store first error term
    yield network.last_epoch
    previous_error = error_delta = network.errors.last()

    epoch = network.last_epoch
    while error_delta > epsilon:
        epoch = epoch + 1
        network.last_epoch += 1

        yield epoch

        last_error = network.errors.last()
        error_delta = abs(last_error - previous_error)
        previous_error = last_error

        if epoch >= max_epochs and error_delta > epsilon:
            logs.message("TRAIN", "Epoch #{} stopped. Network didn't "
                                  "converge after {} iterations"
                                  "".format(epoch, max_epochs))
            return

    if np.isnan(error_delta) or np.isinf(error_delta):
        logs.message("TRAIN", "Epoch #{} stopped. Network error value is "
                              "invalid".format(epoch))
    else:
        logs.message("TRAIN", "Epoch #{} stopped. Network converged."
                              "".format(epoch))


def shuffle(*arrays):
    """ Make a random shuffle for all arrays.

    Parameters
    ----------
    *arrays
        List of arrays that should be shuffled.

    Returns
    -------
    list
        List of arrays that contain shuffeled input data.
    """
    if not arrays:
        return tuple()

    arrays_without_none = [array for array in arrays if array is not None]

    if not arrays_without_none:
        return arrays

    first = arrays_without_none[0]
    n_samples = first.shape[0]

    for array in arrays_without_none:
        if n_samples != array.shape[0]:
            raise ValueError("All matrices should have the same "
                             "number of rows")

    indices = np.arange(n_samples)
    np.random.shuffle(indices)

    arrays = list(arrays)
    for i, array in enumerate(arrays):
        arrays[i] = array[indices] if array is not None else None

    return arrays


def normalize_error(output):
    """ Normalize error output when result is non-scalar.

    Parameters
    ----------
    output : array-like
        Input can be any numpy array or matrix.

    Returns
    -------
    int, float
        Return sum of all absolute values.
    """
    return np.sum(np.abs(output))


def step(input_value):
    """ Step function.
    """
    return np.where(input_value > 0, 1, 0)
